"""
Collection resource access (JSON-LD context and maybe more)
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import sys
import os
import logging
log = logging.getLogger(__name__)

from django.http                        import HttpResponse

from annalist                           import message
from annalist                           import layout

from annalist.views.displayinfo         import DisplayInfo
from annalist.views.generic             import AnnalistGenericView

class SiteResourceAccess(AnnalistGenericView):
    """
    View class for site resource access

    This view class returns a site data resource.
    """

    # NOTE: the logic of this view is modelled on `entityresource`, but currently
    #       the only resource recognized is the JSON-LD context.

    # @TODO: define common superclass with `entityresource` to share common logic.

    def __init__(self):
        super(SiteResourceAccess, self).__init__()
        return

    # GET

    def get(self, request, coll_id=None, resource_ref=None, view_id=None):
        """
        Access specified entity resource
        """
        log.info("SiteResourceAccess.get: resource_ref %s"%(resource_ref,))
        viewinfo = self.view_setup(request.GET.dict())
        if viewinfo.http_response:
            return viewinfo.http_response

        # Locate resource
        resource_info = self.find_resource(viewinfo, resource_ref)
        if resource_info is None:
            return self.error(
                dict(self.error404values(),
                    message=message.SITE_RESOURCE_NOT_DEFINED%
                        { 'ref': resource_ref
                        }
                    )
                )
        resource_file = viewinfo.site.resource_file(resource_info["resource_name"])
        if resource_file is None:
            return self.error(
                dict(self.error404values(),
                    message=message.SITE_RESOURCE_NOT_EXIST%
                        { 'ref': resource_ref
                        }
                    )
                )

        # Return resource
        try:
            response = self.resource_response(resource_file, resource_info["resource_type"])
        except Exception as e:
            log.exception(str(e))
            response = self.error(
                dict(self.error500values(),
                    message=str(e)+" - see server log for details"
                    )
                )
        finally:
            resource_file.close()
        return response

    def view_setup(self, request_dict):
        """
        Assemble display information for view request handler
        """
        action                        = "view"
        self.default_continuation_url = None
        viewinfo = DisplayInfo(self, action, request_dict, self.default_continuation_url)
        viewinfo.get_site_info(self.get_request_host())
        viewinfo.check_authorization(action)
        return viewinfo

    def find_resource(self, viewinfo, resource_ref):
        """
        Return a description for the indicated site resource, or None
        """
        log.info("SiteResourceAccess.find_resource %s"%(resource_ref))
        if resource_ref == layout.SITEDATA_CONTEXT_FILE:
            return (
                { 'resource_type': "application/ld+json"
                , 'resource_name': resource_ref
                })
        return None

    def resource_response(self, resource_file, resource_type):
        """
        Construct response containing body of referenced resource,
        with supplied resoure_type as its content_type
        """
        # @@TODO: assumes response can reasonably be held in memory;
        #         consider 'StreamingHttpResponse'?
        response = HttpResponse(content_type=resource_type)
        response.write(resource_file.read())
        return response

# End.



"""
Annalist site root URL definitions
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

from django.conf                import settings
from django.conf.urls           import patterns, include, url

from django.contrib             import admin
admin.autodiscover()

from annalist.views.home        import AnnalistHomeView
from annalist.views.statichack  import serve_static

urlpatterns = patterns('',
    url(r'^$',         AnnalistHomeView.as_view(), name='AnnalistHomeView'),

    url(r'^admin/',    include(admin.site.urls)),

    url(r'^annalist/', include('annalist.urls')),
    )

if not settings.DEBUG:
    urlpatterns += patterns('',
        url(r'^static/(?P<path>.*)$', serve_static),
        )



"""
Tests for linked record functions.

This test suite uses a setup that is specifically intended to test functions 
that involve links between entities.
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import os
import unittest

import logging
log = logging.getLogger(__name__)

from django.conf                    import settings
from django.contrib.auth.models     import User
from django.test                    import TestCase
from django.core.urlresolvers       import resolve, reverse
from django.test.client             import Client

from annalist.identifiers           import ANNAL

from annalist.models.entity         import EntityRoot, Entity
from annalist.models.site           import Site
from annalist.models.collection     import Collection
from annalist.models.recordtype     import RecordType
from annalist.models.recordlist     import RecordList
from annalist.models.recordview     import RecordView
from annalist.models.recordfield    import RecordField
from annalist.models.entitytypeinfo import EntityTypeInfo

from annalist.views.form_utils.fieldchoice  import FieldChoice

from AnnalistTestCase       import AnnalistTestCase
from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
from entity_testutils       import (
    collection_entity_view_url,
    create_test_user,
    create_user_permissions,
    context_list_entities,
    context_list_head_fields,
    context_list_item_fields
    )
from entity_testentitydata  import (
    entity_url, entitydata_edit_url, entitydata_delete_confirm_url,
    entitydata_list_type_url, entitydata_list_all_url,
    )

#   -----------------------------------------------------------------------------
#
#   Test data
#
#   -----------------------------------------------------------------------------

testsrc_type_create_values = (
    { 'annal:type':                 "annal:Type"
    , 'rdfs:label':                 "testsrc_type label"
    , 'rdfs:comment':               "testsrc_type comment"
    , 'annal:uri':                  "test:type/testsrc_type"
    , 'annal:type_view':            "testsrc_view"
    , 'annal:type_list':            "testsrc_list"
    })

testtgt_type_create_values = (
    { 'annal:type':                 "annal:Type"
    , 'rdfs:label':                 "testtgt_type label"
    , 'rdfs:comment':               "testtgt_type comment"
    , 'annal:uri':                  "test:type/testtgt_type"
    , 'annal:type_view':            "testtgt_view"
    , 'annal:type_list':            "testtgt_list"
    })

testsrc_view_create_values = (
    { 'annal:type':                 "annal:View"
    , 'rdfs:label':                 "testsrc_view label"
    , 'rdfs:comment':               "testsrc_view comment"
    , 'annal:record_type':          ""
    , 'annal:add_field':            "yes"
    , 'annal:view_fields':
      [ { 'annal:field_id':             "Entity_id"
        , 'annal:field_placement':      "small:0,12;medium:0,6"
        }
      , { 'annal:field_id':             "testtgtref_field"
        , 'annal:field_placement':      "small:0,12;medium:6,6"
        }
      , { 'annal:field_id':             "Entity_label"
        , 'annal:field_placement':      "small:0,12"
        }
      , { 'annal:field_id':             "Entity_comment"
        , 'annal:field_placement':      "small:0,12"
        }
      ]
    })

testtgt_view_create_values = (
    { 'annal:type':                 "annal:View"
    , 'rdfs:label':                 "testtgt_view label"
    , 'rdfs:comment':               "testtgt_view commemnt"
    , 'annal:record_type':          ""
    , 'annal:add_field':            "yes"
    , 'annal:view_fields':
      [ { 'annal:field_id':             "Entity_id"
        , 'annal:field_placement':      "small:0,12;medium:0,6"
        }
      , { 'annal:field_id':             "Entity_label"
        , 'annal:field_placement':      "small:0,12"
        }
      , { 'annal:field_id':             "Entity_comment"
        , 'annal:field_placement':      "small:0,12"
        }
      ]
    })

testsrc_list_create_values = (
    { 'annal:type':                 "annal:List"
    , 'rdfs:label':                 "testsrc_list label"
    , 'rdfs:comment':               "testsrc_list commemnt"
    , 'annal:record_type':          ""
    , "annal:display_type":         "List"
    , "annal:default_view":         "testsrc_view"
    , "annal:default_type":         "testsrc_type"
    , "annal:list_entity_selector": "test:type/testsrc_type in [@type]"
    , "annal:list_fields":
      [ { "annal:field_id":             "Entity_id"
        , "annal:field_placement":      "small:0,3"
        }
      , { "annal:field_id":             "testtgtref_field"
        , "annal:field_placement":      "small:3,3"
        }
      , { "annal:field_id":             "Entity_label"
        , "annal:field_placement":      "small:6,6"
        }
      ]
    })

testtgt_list_create_values = (
    { 'annal:type':                 "annal:List"
    , 'rdfs:label':                 "testtgt_list label"
    , 'rdfs:comment':               "testtgt_list commemnt"
    , 'annal:record_type':          ""
    , "annal:display_type":         "List"
    , "annal:default_view":         "testtgt_view"
    , "annal:default_type":         "testtgt_type"
    , "annal:list_entity_selector": "test:type/testtgt_type in [@type]"
    , "annal:list_fields":
      [ { "annal:field_id":             "Entity_id"
        , "annal:field_placement":      "small:0,3"
        }
      , { "annal:field_id":             "Entity_label"
        , "annal:field_placement":      "small:3,9"
        }
      ]
    })

testtgtref_field_create_values = (
    { 'annal:type':                 "annal:Field"
    , 'rdfs:label':                 "testtgtref_field label"
    , 'rdfs:comment':               "testtgtref_field comment"
    , 'annal:property_uri':         "test:testtgtref"
    , 'annal:field_entity_type':    "test:type/testsrc_type"    # Field-ref domain type
    , 'annal:field_value_type':     "test:type/testtgt_type"    # Field-ref range type
    , 'annal:field_render_type':    "Enum"
    , 'annal:field_ref_type':       "testtgt_type"
    , 'annal:placeholder':          "(reference to testtgt_type entity)"
    , 'annal:default_value':        ""
    })

def testsrc_entity_create_values(entity_id, tgtref_id):
    return (
        { 'rdfs:label':                 "testsrc_entity %s label"%entity_id
        , 'rdfs:comment':               "testsrc_entity %s comment"%entity_id
        , 'test:testtgtref':            "testtgt_type/"+tgtref_id
        })

def testtgt_entity_create_values(entity_id):
    return (
        { 'rdfs:label':                 "testtgt_entity %s label"%entity_id
        , 'rdfs:comment':               "testtgt_entity %s comment"%entity_id
        })

#   -----------------------------------------------------------------------------
#
#   Linked record tests
#
#   -----------------------------------------------------------------------------

class LinkedRecordTest(AnnalistTestCase):
    """
    Tests for linked records
    """

    def setUp(self):
        init_annalist_test_site()
        init_annalist_test_coll()
        self.testsite    = Site(TestBaseUri, TestBaseDir)
        self.testcoll    = Collection(self.testsite, "testcoll")
        # Populate collection with linked record types, views and lists
        self.testsrc_type = RecordType.create(self.testcoll, "testsrc_type", testsrc_type_create_values)
        self.testtgt_type = RecordType.create(self.testcoll, "testtgt_type", testtgt_type_create_values)
        self.testsrc_view = RecordView.create(self.testcoll, "testsrc_view", testsrc_view_create_values)
        self.testtgt_view = RecordView.create(self.testcoll, "testtgt_view", testtgt_view_create_values)
        self.testsrc_list = RecordList.create(self.testcoll, "testsrc_list", testsrc_list_create_values)
        self.testtgt_list = RecordList.create(self.testcoll, "testtgt_list", testtgt_list_create_values)
        self.testtgtref_field = RecordField.create(self.testcoll, "testtgtref_field", testtgtref_field_create_values)
        self.no_options   = [ FieldChoice('', label="(no options)") ]
        self.tgt_options  = (
            [ FieldChoice("testtgt_type/"+v, 
                label="testtgt_entity %s label"%v,
                link=entity_url("testcoll", "testtgt_type", v))
              for v in ["testtgt1", "testtgt2"]
            ])
        # Create data records for testing:
        self.testtgt_type_info = EntityTypeInfo(self.testsite, self.testcoll, "testtgt_type", create_typedata=True)
        self.testsrc_type_info = EntityTypeInfo(self.testsite, self.testcoll, "testsrc_type", create_typedata=True)
        for tgt_id in ("testtgt1", "testtgt2"):
            self.testtgt_type_info.create_entity(tgt_id, testtgt_entity_create_values(tgt_id))
        for src_id, tgt_ref in (("testsrc1", "testtgt1"), ("testsrc2", "testtgt2")):
            self.testsrc_type_info.create_entity(src_id, testsrc_entity_create_values(src_id, tgt_ref))
        # Login and permissions
        create_test_user(self.testcoll, "testuser", "testpassword")
        self.client = Client(HTTP_HOST=TestHost)
        loggedin = self.client.login(username="testuser", password="testpassword")
        self.assertTrue(loggedin)
        return

    def tearDown(self):
        # resetSitedata(scope="collections")
        return

    @classmethod
    def tearDownClass(cls):
        resetSitedata()
        return

    # Utility functions

    # Tests

    def test_view_entity_references(self):
        u = entity_url(coll_id="testcoll", type_id="testsrc_type", entity_id="testsrc1")
        r = self.client.get(u)
        self.assertEqual(r.status_code,   200)
        self.assertEqual(r.reason_phrase, "OK")
        # Test context
        self.assertEqual(len(r.context['fields']), 4)
        i = 0
        self.assertEqual(r.context['fields'][i].field_id,         "Entity_id")
        self.assertEqual(r.context['fields'][i].field_value,      "testsrc1")
        self.assertEqual(r.context['fields'][i].field_value_link, None)
        self.assertEqual(r.context['fields'][i].options,          self.no_options)
        i = 1
        self.assertEqual(r.context['fields'][i].field_id,         "testtgtref_field")
        self.assertEqual(r.context['fields'][i].field_value,      "testtgt_type/testtgt1")
        self.assertEqual(r.context['fields'][i].field_value_link, "/testsite/c/testcoll/d/testtgt_type/testtgt1/")
        self.assertEqual(r.context['fields'][i].options,          self.tgt_options)
        i = 2
        self.assertEqual(r.context['fields'][i].field_id,         "Entity_label")
        self.assertEqual(r.context['fields'][i].field_value,      "testsrc_entity testsrc1 label")
        self.assertEqual(r.context['fields'][i].field_value_link, None)
        self.assertEqual(r.context['fields'][i].options,          self.no_options)
        i = 3
        self.assertEqual(r.context['fields'][i].field_id,         "Entity_comment")
        self.assertEqual(r.context['fields'][i].field_value,      "testsrc_entity testsrc1 comment")
        self.assertEqual(r.context['fields'][i].field_value_link, None)
        self.assertEqual(r.context['fields'][i].options,          self.no_options)
        return

    def test_list_entity_references(self):
        # List linked records - check values in listing
        u = entitydata_list_type_url("testcoll", "testsrc_type", list_id="testsrc_list")
        r = self.client.get(u)
        self.assertEqual(r.status_code,   200)
        self.assertEqual(r.reason_phrase, "OK")
        # Test context
        entities    = context_list_entities(r.context)
        head_fields = context_list_head_fields(r.context)
        self.assertEqual(len(entities),    2)
        self.assertEqual(len(head_fields), 3)
        entity_values = (
            (entities[0], "testsrc1", "testtgt1"), 
            (entities[1], "testsrc2", "testtgt2")
            )
        for entc, esrc, etgt in entity_values:
            item_fields = context_list_item_fields(r.context, entc)
            self.assertEqual(len(item_fields), 3)
            self.assertEqual(entc['entity_id'],               esrc)
            self.assertEqual(entc['entity_type_id'],          "testsrc_type")
            self.assertEqual(item_fields[0].field_id,         "Entity_id")
            self.assertEqual(item_fields[0].field_value,      esrc)
            self.assertEqual(item_fields[0].field_value_link, None)
            self.assertEqual(item_fields[1].field_id,         "testtgtref_field")
            self.assertEqual(item_fields[1].field_value,      "testtgt_type/"+etgt)
            self.assertEqual(item_fields[1].field_value_link, "/testsite/c/testcoll/d/testtgt_type/%s/"%etgt)
            self.assertEqual(item_fields[2].field_id,         "Entity_label")
            self.assertEqual(item_fields[2].field_value,      "testsrc_entity %s label"%esrc)
            self.assertEqual(item_fields[2].field_value_link, None)
        return

# End.



"""
Tests for authorization functions
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import os
import unittest

import logging
log = logging.getLogger(__name__)

from django.conf                    import settings
from django.contrib.auth.models     import User
from django.test                    import TestCase
from django.core.urlresolvers       import resolve, reverse
from django.test.client             import Client

from utils.SuppressLoggingContext   import SuppressLogging

from annalist.identifiers           import ANNAL

from annalist.models.entity         import EntityRoot, Entity
from annalist.models.annalistuser   import AnnalistUser
from annalist.models.site           import Site
from annalist.models.collection     import Collection
from annalist.models.recordtype     import RecordType
from annalist.models.recordtypedata import RecordTypeData
from annalist.models.entitydata     import EntityData

from AnnalistTestCase       import AnnalistTestCase
from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
from entity_testutils       import (
    create_user_permissions,
    collection_new_form_data, collection_remove_form_data,
    context_list_entities,
    context_list_item_fields, context_list_item_field_value
    )
from entity_testentitydata  import (
    # recorddata_dir,  entitydata_dir,
    entity_url, entitydata_edit_url, entitydata_delete_confirm_url,
    entitydata_list_type_url, entitydata_list_all_url,
    entitydata_value_keys, entitydata_create_values, entitydata_values, 
    entitydata_context_data, entitydata_form_data, 
    entitydata_delete_form_data,
    entitydata_delete_confirm_form_data
    )
from entity_testuserdata    import (
    annalistuser_dir,
    annalistuser_site_url, annalistuser_coll_url, annalistuser_url, annalistuser_edit_url,
    annalistuser_value_keys, annalistuser_load_keys,
    annalistuser_create_values, annalistuser_values, annalistuser_read_values,
    annalistuser_view_form_data,
    annalistuser_delete_form_data,
    annalistuser_delete_confirm_form_data
    )
from entity_testtypedata    import (
    recordtype_create_values, 
    recordtype_entity_view_form_data,
    recordtype_delete_form_data,
    recordtype_delete_confirm_form_data
    )

#   -----------------------------------------------------------------------------
#
#   Authorization tests
#
#   -----------------------------------------------------------------------------

class AuthorizationTest(AnnalistTestCase):
    """
    Tests for authorization (access control) logic
    """

    def setUp(self):
        init_annalist_test_site()
        init_annalist_test_coll()
        self.testsite = Site(TestBaseUri, TestBaseDir)
        self.testcoll = Collection(self.testsite, "testcoll")
        # populate site with different classes of users: admin, config, create, update, delete, view
        self.user_admin  = AnnalistUser.create(self.testcoll, "user_admin", 
            annalistuser_create_values(
                coll_id="testcoll", user_id="user_admin",
                user_name="Admin User",
                user_uri="mailto:user_admin@%s"%TestHost, 
                user_permissions=["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
                )
            )
        self.user_config  = AnnalistUser.create(self.testcoll, "user_config", 
            annalistuser_create_values(
                coll_id="testcoll", user_id="user_config",
                user_name="Admin User",
                user_uri="mailto:user_config@%s"%TestHost, 
                user_permissions=["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG"]
                )
            )
        self.user_create  = AnnalistUser.create(self.testcoll, "user_create", 
            annalistuser_create_values(
                coll_id="testcoll", user_id="user_create",
                user_name="Admin User",
                user_uri="mailto:user_create@%s"%TestHost, 
                user_permissions=["VIEW", "UPDATE", "CREATE"]
                )
            )
        self.user_update  = AnnalistUser.create(self.testcoll, "user_update", 
            annalistuser_create_values(
                coll_id="testcoll", user_id="user_update",
                user_name="Admin User",
                user_uri="mailto:user_update@%s"%TestHost, 
                user_permissions=["VIEW", "UPDATE"]
                )
            )
        self.user_delete  = AnnalistUser.create(self.testcoll, "user_delete", 
            annalistuser_create_values(
                coll_id="testcoll", user_id="user_delete",
                user_name="Admin User",
                user_uri="mailto:user_delete@%s"%TestHost, 
                user_permissions=["VIEW", "UPDATE", "DELETE"]
                )
            )
        self.user_view  = AnnalistUser.create(self.testcoll, "user_view", 
            annalistuser_create_values(
                coll_id="testcoll", user_id="user_view",
                user_name="Admin User",
                user_uri="mailto:user_view@%s"%TestHost, 
                user_permissions=["VIEW"]
                )
            )
        self.user_site_admin  = AnnalistUser.create(
            self.testsite, 
            "user_site_admin", 
            annalistuser_create_values(
                coll_id="testcoll", user_id="user_site_admin",
                user_name="Site_admin User",
                user_uri="mailto:user_site_admin@%s"%TestHost, 
                user_permissions=["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
                ),
            use_altpath=True
            )
        self.user_site_create_coll  = AnnalistUser.create(
            self.testsite, 
            "user_site_create_coll", 
            annalistuser_create_values(
                coll_id="testcoll", user_id="user_site_create_coll",
                user_name="Site_create User",
                user_uri="mailto:user_site_create_coll@%s"%TestHost, 
                user_permissions=["VIEW", "CREATE_COLLECTION"]
                ),
            use_altpath=True
            )
        self.user_site_delete_coll  = AnnalistUser.create(
            self.testsite, 
            "user_site_delete_coll", 
            annalistuser_create_values(
                coll_id="testcoll", user_id="user_site_delete_coll_coll",
                user_name="Site_delete User",
                user_uri="mailto:user_site_delete_coll@%s"%TestHost, 
                user_permissions=["VIEW", "CREATE_COLLECTION", "DELETE_COLLECTION"]
                ),
            use_altpath=True
            )
        self.user_site_view  = AnnalistUser.create(
            self.testsite, 
            "user_site_view", 
            annalistuser_create_values(
                coll_id="testcoll", user_id="user_site_view",
                user_name="Site_view User",
                user_uri="mailto:user_site_view@%s"%TestHost, 
                user_permissions=["VIEW"]
                ),
            use_altpath=True
            )
        return

    def tearDown(self):
        resetSitedata(scope="collections")
        return

    @classmethod
    def tearDownClass(cls):
        resetSitedata()
        return

    # Utility functions

    def login_user(self, user_id):
        # Create user identity and log in (authenticate).  Updates the test client object.
        self.user = User.objects.create_user(user_id, '%s@%s'%(user_id, TestHost), 'testpassword')
        self.user.save()
        self.client = Client(HTTP_HOST=TestHost)
        loggedin = self.client.login(username=user_id, password="testpassword")
        self.assertTrue(loggedin)
        return

    # View site home page
    def view_site_home_page(self):
        u = reverse("AnnalistSiteView")
        r = self.client.get(u)
        return r

    # Create collection: requires site-level CREATE permission

    def create_collection(self):
        u = reverse("AnnalistSiteView")
        f = collection_new_form_data("testnew")
        r = self.client.post(u, f)
        return r

    # Remove collection: requires site-level DELETE permission

    def remove_collection(self):
        u = reverse("AnnalistSiteView")
        f = collection_remove_form_data(["coll1", "coll3"])
        r = self.client.post(u, f)
        return r

    # User access - requires ADMIN permission

    def create_user(self, user_id):
        # Create placeholder for testing
        return create_user_permissions(
            self.testcoll, user_id, 
            ["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
            )

    def list_users(self):
        # requires ADMIN
        u = entitydata_list_type_url(
            coll_id="testcoll", type_id="_user", 
            list_id="Default_list"
            )
        r = self.client.get(u)
        return r

    def view_user_get(self):
        # requires ADMIN
        e = self.create_user("view_user")
        u = entitydata_edit_url(action="view", 
            coll_id="testcoll", type_id="_user", entity_id="view_user", 
            view_id="Default_view"
            )
        r = self.client.get(u)
        return r

    def view_user_edit(self):
        # requires ADMIN
        e = self.create_user("view_user")
        u = entitydata_edit_url(action="view", 
            coll_id="testcoll", type_id="_user", entity_id="view_user", 
            view_id="Default_view"
            )
        f = annalistuser_view_form_data(action="view",
            coll_id="testcoll", user_id="view_user",
            user_name="View User",
            user_uri="mailto:view_user@%s"%(TestHost), 
            user_permissions=["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"],
            edit="edit"
            )
        r = self.client.post(u, f)
        return r

    def view_user_copy(self):
        # requires ADMIN
        e = self.create_user("view_user")
        u = entitydata_edit_url(action="view", 
            coll_id="testcoll", type_id="_user", entity_id="view_user", 
            view_id="Default_view"
            )
        f = annalistuser_view_form_data(action="view",
            coll_id="testcoll", user_id="view_user",
            user_name="View User",
            user_uri="mailto:view_user@%s"%(TestHost), 
            user_permissions=["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"],
            copy="copy"
            )
        r = self.client.post(u, f)
        return r

    def new_user(self):
        # requires ADMIN
        u = entitydata_edit_url(action="new", 
            coll_id="testcoll", type_id="_user",
            view_id="Default_view"
            )
        f = annalistuser_view_form_data(action="new",
            coll_id="testcoll", user_id="new_user",
            user_name="New User",
            user_uri="mailto:new_user@%s"%(TestHost), 
            user_permissions=["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
            )
        r = self.client.post(u, f)
        return r

    def copy_user(self):
        # requires ADMIN
        e = self.create_user("copy_user")
        u = entitydata_edit_url(action="copy", 
            coll_id="testcoll", type_id="_user", entity_id="copy_user", 
            view_id="Default_view"
            )
        f = annalistuser_view_form_data(action="copy",
            coll_id="testcoll", user_id="copy_user_new", orig_id="copy_user",
            user_name="Copy User",
            user_uri="mailto:copy_user_new@%s"%(TestHost), 
            user_permissions=["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
            )
        r = self.client.post(u, f)
        return r

    def edit_user(self):
        e = self.create_user("edit_user")
        u = entitydata_edit_url(action="edit", 
            coll_id="testcoll", type_id="_user", entity_id="edit_user", 
            view_id="Default_view"
            )
        f = annalistuser_view_form_data(action="edit",
            coll_id="testcoll", user_id="edit_user",
            user_name="Edit User",
            user_uri="mailto:edit_user@%s"%(TestHost), 
            user_permissions=["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
            )
        r = self.client.post(u, f)
        return r

    def delete_user(self):
        # requires ADMIN
        e = self.create_user("delete_user")
        u = entitydata_list_type_url(
            coll_id="testcoll", type_id="_user", 
            list_id="Default_list"
            )
        f = annalistuser_delete_form_data(user_id="delete_user")
        r = self.client.post(u, f)
        # log.info(r.content)
        return r

    def delete_user_confirmed(self):
        # requires ADMIN
        e = self.create_user("delete_user")
        u = entitydata_delete_confirm_url(coll_id="testcoll", type_id="_user")
        f = annalistuser_delete_confirm_form_data(user_id="delete_user")
        r = self.client.post(u, f)
        return r

    # Type access - requires VIEW/CONFIG permission

    def create_type(self, type_id):
        # Create placeholder for testing
        t = RecordType.create(self.testcoll, type_id, 
            recordtype_create_values(coll_id="testcoll", type_id="testtype")
            )
        return t

    def list_types(self):
        # requires VIEW
        u = entitydata_list_type_url(
            coll_id="testcoll", type_id="_type", 
            list_id="Default_list"
            )
        r = self.client.get(u)
        return r

    def view_type_get(self):
        # requires VIEW
        e = self.create_type("view_type")
        u = entitydata_edit_url(action="view", 
            coll_id="testcoll", type_id="_type", entity_id="view_type", 
            view_id="Default_view"
            )
        r = self.client.get(u)
        return r

    def view_type_edit(self):
        # requires CONFIG
        e = self.create_type("view_type")
        u = entitydata_edit_url(action="view", 
            coll_id="testcoll", type_id="_type", entity_id="view_type", 
            view_id="Default_view"
            )
        f = recordtype_entity_view_form_data(action="view",
            coll_id="testcoll", type_id="_type", orig_id="view_type",
            edit="edit"
            )
        r = self.client.post(u, f)
        return r

    def view_type_copy(self):
        # requires CONFIG
        e = self.create_type("view_type")
        u = entitydata_edit_url(action="view", 
            coll_id="testcoll", type_id="_type", entity_id="view_type", 
            view_id="Default_view"
            )
        f = recordtype_entity_view_form_data(action="view",
            coll_id="testcoll", type_id="_type", orig_id="view_type",
            copy="copy"
            )
        r = self.client.post(u, f)
        return r

    def new_type(self):
        # requires CONFIG
        u = entitydata_edit_url(action="new", 
            coll_id="testcoll", type_id="_type",
            view_id="Default_view"
            )
        f = recordtype_entity_view_form_data(action="new",
            coll_id="testcoll", 
            type_id="new_type", orig_id="orig_type"
            )
        r = self.client.post(u, f)
        return r

    def copy_type(self):
        # requires CONFIG
        e = self.create_type("copy_type")
        u = entitydata_edit_url(action="copy", 
            coll_id="testcoll", type_id="_type", entity_id="copy_type", 
            view_id="Default_view"
            )
        f = recordtype_entity_view_form_data(action="copy",
            coll_id="testcoll", 
            type_id="copy_type_new", orig_id="copy_type"
            )
        r = self.client.post(u, f)
        return r

    def edit_type(self):
        # requires CONFIG
        e = self.create_type("edit_type")
        u = entitydata_edit_url(action="edit", 
            coll_id="testcoll", type_id="_type", entity_id="edit_type", 
            view_id="Default_view"
            )
        f = recordtype_entity_view_form_data(action="edit",
            coll_id="testcoll", 
            type_id="edit_type"
            )
        r = self.client.post(u, f)
        return r

    def delete_type(self):
        # requires CONFIG
        e = self.create_type("delete_type")
        u = entitydata_list_type_url(
            coll_id="testcoll", type_id="_type", 
            list_id="Default_list"
            )
        f = recordtype_delete_form_data(type_id="delete_type")
        # log.info("u %s"%u)
        # log.info("f %r"%f)
        r = self.client.post(u, f)
        # log.info(r.content)
        return r

    def delete_type_confirmed(self):
        # requires CONFIG
        e = self.create_type("delete_type")
        u = entitydata_delete_confirm_url(coll_id="testcoll", type_id="_type")
        f = entitydata_delete_confirm_form_data(entity_id="delete_type")
        r = self.client.post(u, f)
        return r

    # Data access - requires VIEW/CREATE/UPDATE/DELETE permissions

    def create_data(self, entity_id):
        # Create placeholder for testing
        typedata = RecordTypeData.create(self.testcoll, "Default_type", {})
        assert typedata is not None
        e = EntityData.create(typedata, entity_id, 
            entitydata_create_values(entity_id, coll_id="testcoll", type_id="Default_type")
            )
        return e

    def list_data(self):
        # requires VIEW
        u = entitydata_list_type_url(
            coll_id="testcoll", type_id="Default_type", 
            list_id="Default_list"
            )
        r = self.client.get(u)
        return r

    def view_data_get(self):
        # requires VIEW
        e = self.create_data("view_data")
        u = entitydata_edit_url(action="view", 
            coll_id="testcoll", type_id="Default_type", entity_id="view_data", 
            view_id="Default_view"
            )
        r = self.client.get(u)
        return r

    def view_data_edit(self):
        # requires UPDATE
        e = self.create_data("view_data")
        u = entitydata_edit_url(action="view", 
            coll_id="testcoll", type_id="Default_type", entity_id="view_data", 
            view_id="Default_view"
            )
        f = entitydata_form_data(action="edit",
            coll_id="testcoll", type_id="Default_data", 
            entity_id="view_data", orig_id="view_data",
            edit="edit"
            )
        r = self.client.post(u, f)
        return r

    def view_data_copy(self):
        # requires CREATE
        e = self.create_data("view_data")
        u = entitydata_edit_url(action="view", 
            coll_id="testcoll", type_id="Default_type", entity_id="view_data", 
            view_id="Default_view"
            )
        f = entitydata_form_data(action="copy",
            coll_id="testcoll", type_id="Default_data", 
            entity_id="copy_view_data", orig_id="view_data",
            copy="copy"
            )
        r = self.client.post(u, f)
        return r

    def new_data(self):
        # requires CREATE
        u = entitydata_edit_url(action="new", 
            coll_id="testcoll", type_id="Default_type",
            view_id="Default_view"
            )
        f = entitydata_form_data(action="new",
            coll_id="testcoll", type_id="Default_type", entity_id="new_entity"
            )
        r = self.client.post(u, f)
        return r

    def copy_data(self):
        # requires CREATE
        e = self.create_data("copy_entity")
        u = entitydata_edit_url(action="copy", 
            coll_id="testcoll", type_id="Default_type", entity_id="copy_entity", 
            view_id="Default_view"
            )
        f = entitydata_form_data(action="copy",
            coll_id="testcoll", type_id="Default_type", 
            entity_id="copy_entity_new", orig_id="copy_entity"
            )
        r = self.client.post(u, f)
        # log.info("r %s"%r)
        return r

    def edit_data(self):
        # requires UPDATE
        e = self.create_data("edit_entity")
        u = entitydata_edit_url(action="edit", 
            coll_id="testcoll", type_id="Default_type", entity_id="edit_entity", 
            view_id="Default_view"
            )
        f = entitydata_form_data(action="edit",
            coll_id="testcoll", type_id="Default_type", entity_id="edit_entity"
            )
        r = self.client.post(u, f)
        # log.info("r %s"%r)
        return r

    def delete_data(self):
        # requires DELETE
        e = self.create_data("delete_entity")
        u = entitydata_list_type_url(
            coll_id="testcoll", type_id="Default_type", 
            list_id="Default_list"
            )
        f = entitydata_delete_form_data(entity_id="delete_entity")
        # log.info("u %s"%u)
        # log.info("f %r"%f)
        r = self.client.post(u, f)
        # log.info(r)
        return r

    def delete_data_confirmed(self):
        # requires DELETE
        e = self.create_data("delete_entity")
        u = entitydata_delete_confirm_url(coll_id="testcoll", type_id="Default_type")
        f = entitydata_delete_confirm_form_data(entity_id="delete_entity")
        r = self.client.post(u, f)
        return r

    # Tests

    def test_bad_user_id(self):
        self.login_user("user-bad-name")
        with SuppressLogging(logging.WARNING):
            self.assertEqual(self.view_site_home_page().status_code, 400)
        return

    def test_admin_user(self):
        self.login_user("user_admin")
        # try each function, test result
        self.assertEqual(self.view_site_home_page().status_code,    200)
        self.assertEqual(self.create_collection().status_code,      403)
        self.assertEqual(self.remove_collection().status_code,      403)
        #
        self.assertEqual(self.list_users().status_code,             200)
        self.assertEqual(self.view_user_get().status_code,          200)
        self.assertEqual(self.view_user_edit().status_code,         302)
        self.assertEqual(self.view_user_copy().status_code,         302)
        self.assertEqual(self.new_user().status_code,               302)
        self.assertEqual(self.copy_user().status_code,              302)
        self.assertEqual(self.edit_user().status_code,              302)
        self.assertEqual(self.delete_user().status_code,            200)
        self.assertEqual(self.delete_user_confirmed().status_code,  302)
        #
        self.assertEqual(self.list_types().status_code,             200)
        self.assertEqual(self.view_type_get().status_code,          200)
        self.assertEqual(self.view_type_edit().status_code,         302)
        self.assertEqual(self.view_type_copy().status_code,         302)
        self.assertEqual(self.new_type().status_code,               302)
        self.assertEqual(self.copy_type().status_code,              302)
        self.assertEqual(self.edit_type().status_code,              302)
        self.assertEqual(self.delete_type().status_code,            200)
        self.assertEqual(self.delete_type_confirmed().status_code,  302)
        #
        self.assertEqual(self.list_data().status_code,              200)
        self.assertEqual(self.view_data_get().status_code,          200)
        self.assertEqual(self.view_data_edit().status_code,         302)
        self.assertEqual(self.view_data_copy().status_code,         302)
        self.assertEqual(self.new_data().status_code,               302)
        self.assertEqual(self.copy_data().status_code,              302)
        self.assertEqual(self.edit_data().status_code,              302)
        self.assertEqual(self.delete_data().status_code,            200)
        self.assertEqual(self.delete_data_confirmed().status_code,  302)
        return

    def test_config_user(self):
        self.login_user("user_config")
        # try each function, test result
        self.assertEqual(self.view_site_home_page().status_code,    200)
        self.assertEqual(self.create_collection().status_code,      403)
        self.assertEqual(self.remove_collection().status_code,      403)
        #
        self.assertEqual(self.list_users().status_code,             403)
        self.assertEqual(self.view_user_get().status_code,          403)
        self.assertEqual(self.view_user_edit().status_code,         403)
        self.assertEqual(self.view_user_copy().status_code,         403)
        self.assertEqual(self.new_user().status_code,               403)
        self.assertEqual(self.copy_user().status_code,              403)
        self.assertEqual(self.edit_user().status_code,              403)
        self.assertEqual(self.delete_user().status_code,            403)
        self.assertEqual(self.delete_user_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_types().status_code,             200)
        self.assertEqual(self.view_type_get().status_code,          200)
        self.assertEqual(self.view_type_edit().status_code,         302)
        self.assertEqual(self.view_type_copy().status_code,         302)
        self.assertEqual(self.new_type().status_code,               302)
        self.assertEqual(self.copy_type().status_code,              302)
        self.assertEqual(self.edit_type().status_code,              302)
        self.assertEqual(self.delete_type().status_code,            200)
        self.assertEqual(self.delete_type_confirmed().status_code,  302)
        #
        self.assertEqual(self.list_data().status_code,              200)
        self.assertEqual(self.view_data_get().status_code,          200)
        self.assertEqual(self.view_data_edit().status_code,         302)
        self.assertEqual(self.view_data_copy().status_code,         302)
        self.assertEqual(self.new_data().status_code,               302)
        self.assertEqual(self.copy_data().status_code,              302)
        self.assertEqual(self.edit_data().status_code,              302)
        self.assertEqual(self.delete_data().status_code,            200)
        self.assertEqual(self.delete_data_confirmed().status_code,  302)
        return

    def test_create_user(self):
        self.login_user("user_create")
        # try each function, test result
        self.assertEqual(self.view_site_home_page().status_code,    200)
        self.assertEqual(self.create_collection().status_code,      403)
        self.assertEqual(self.remove_collection().status_code,      403)
        #
        self.assertEqual(self.list_users().status_code,             403)
        self.assertEqual(self.view_user_get().status_code,          403)
        self.assertEqual(self.view_user_edit().status_code,         403)
        self.assertEqual(self.view_user_copy().status_code,         403)
        self.assertEqual(self.new_user().status_code,               403)
        self.assertEqual(self.copy_user().status_code,              403)
        self.assertEqual(self.edit_user().status_code,              403)
        self.assertEqual(self.delete_user().status_code,            403)
        self.assertEqual(self.delete_user_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_types().status_code,             200)
        self.assertEqual(self.view_type_get().status_code,          200)
        self.assertEqual(self.view_type_edit().status_code,         403)
        self.assertEqual(self.view_type_copy().status_code,         403)
        self.assertEqual(self.new_type().status_code,               403)
        self.assertEqual(self.copy_type().status_code,              403)
        self.assertEqual(self.edit_type().status_code,              403)
        self.assertEqual(self.delete_type().status_code,            403)
        self.assertEqual(self.delete_type_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_data().status_code,              200)
        self.assertEqual(self.view_data_get().status_code,          200)
        self.assertEqual(self.view_data_edit().status_code,         302)
        self.assertEqual(self.view_data_copy().status_code,         302)
        self.assertEqual(self.new_data().status_code,               302)
        self.assertEqual(self.copy_data().status_code,              302)
        self.assertEqual(self.edit_data().status_code,              302)
        self.assertEqual(self.delete_data().status_code,            403)
        self.assertEqual(self.delete_data_confirmed().status_code,  403)
        return

    def test_update_user(self):
        self.login_user("user_update")
        # try each function, test result
        self.assertEqual(self.view_site_home_page().status_code,    200)
        self.assertEqual(self.create_collection().status_code,      403)
        self.assertEqual(self.remove_collection().status_code,      403)
        #
        self.assertEqual(self.list_users().status_code,             403)
        self.assertEqual(self.view_user_get().status_code,          403)
        self.assertEqual(self.view_user_edit().status_code,         403)
        self.assertEqual(self.view_user_copy().status_code,         403)
        self.assertEqual(self.new_user().status_code,               403)
        self.assertEqual(self.copy_user().status_code,              403)
        self.assertEqual(self.edit_user().status_code,              403)
        self.assertEqual(self.delete_user().status_code,            403)
        self.assertEqual(self.delete_user_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_types().status_code,             200)
        self.assertEqual(self.view_type_get().status_code,          200)
        self.assertEqual(self.view_type_edit().status_code,         403)
        self.assertEqual(self.view_type_copy().status_code,         403)
        self.assertEqual(self.new_type().status_code,               403)
        self.assertEqual(self.copy_type().status_code,              403)
        self.assertEqual(self.edit_type().status_code,              403)
        self.assertEqual(self.delete_type().status_code,            403)
        self.assertEqual(self.delete_type_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_data().status_code,              200)
        self.assertEqual(self.view_data_get().status_code,          200)
        self.assertEqual(self.view_data_edit().status_code,         302)
        self.assertEqual(self.view_data_copy().status_code,         403)
        self.assertEqual(self.new_data().status_code,               403)
        self.assertEqual(self.copy_data().status_code,              403)
        self.assertEqual(self.edit_data().status_code,              302)
        self.assertEqual(self.delete_data().status_code,            403)
        self.assertEqual(self.delete_data_confirmed().status_code,  403)
        return

    def test_delete_user(self):
        self.login_user("user_delete")
        # try each function, test result
        self.assertEqual(self.view_site_home_page().status_code,    200)
        self.assertEqual(self.create_collection().status_code,      403)
        self.assertEqual(self.remove_collection().status_code,      403)
        #
        self.assertEqual(self.list_users().status_code,             403)
        self.assertEqual(self.view_user_get().status_code,          403)
        self.assertEqual(self.view_user_edit().status_code,         403)
        self.assertEqual(self.view_user_copy().status_code,         403)
        self.assertEqual(self.new_user().status_code,               403)
        self.assertEqual(self.copy_user().status_code,              403)
        self.assertEqual(self.edit_user().status_code,              403)
        self.assertEqual(self.delete_user().status_code,            403)
        self.assertEqual(self.delete_user_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_types().status_code,             200)
        self.assertEqual(self.view_type_get().status_code,          200)
        self.assertEqual(self.view_type_edit().status_code,         403)
        self.assertEqual(self.view_type_copy().status_code,         403)
        self.assertEqual(self.new_type().status_code,               403)
        self.assertEqual(self.copy_type().status_code,              403)
        self.assertEqual(self.edit_type().status_code,              403)
        self.assertEqual(self.delete_type().status_code,            403)
        self.assertEqual(self.delete_type_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_data().status_code,              200)
        self.assertEqual(self.view_data_get().status_code,          200)
        self.assertEqual(self.view_data_edit().status_code,         302)
        self.assertEqual(self.view_data_copy().status_code,         403)
        self.assertEqual(self.new_data().status_code,               403)
        self.assertEqual(self.copy_data().status_code,              403)
        self.assertEqual(self.edit_data().status_code,              302)
        self.assertEqual(self.delete_data().status_code,            200)
        self.assertEqual(self.delete_data_confirmed().status_code,  302)
        return

    def test_view_user(self):
        self.login_user("user_view")
        # try each function, test result
        self.assertEqual(self.view_site_home_page().status_code,    200)
        self.assertEqual(self.create_collection().status_code,      403)
        self.assertEqual(self.remove_collection().status_code,      403)
        #
        self.assertEqual(self.list_users().status_code,             403)
        self.assertEqual(self.view_user_get().status_code,          403)
        self.assertEqual(self.view_user_edit().status_code,         403)
        self.assertEqual(self.view_user_copy().status_code,         403)
        self.assertEqual(self.new_user().status_code,               403)
        self.assertEqual(self.copy_user().status_code,              403)
        self.assertEqual(self.edit_user().status_code,              403)
        self.assertEqual(self.delete_user().status_code,            403)
        self.assertEqual(self.delete_user_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_types().status_code,             200)
        self.assertEqual(self.view_type_get().status_code,          200)
        self.assertEqual(self.view_type_edit().status_code,         403)
        self.assertEqual(self.view_type_copy().status_code,         403)
        self.assertEqual(self.new_type().status_code,               403)
        self.assertEqual(self.copy_type().status_code,              403)
        self.assertEqual(self.edit_type().status_code,              403)
        self.assertEqual(self.delete_type().status_code,            403)
        self.assertEqual(self.delete_type_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_data().status_code,              200)
        self.assertEqual(self.view_data_get().status_code,          200)
        self.assertEqual(self.view_data_edit().status_code,         403)
        self.assertEqual(self.view_data_copy().status_code,         403)
        self.assertEqual(self.new_data().status_code,               403)
        self.assertEqual(self.copy_data().status_code,              403)
        self.assertEqual(self.edit_data().status_code,              403)
        self.assertEqual(self.delete_data().status_code,            403)
        self.assertEqual(self.delete_data_confirmed().status_code,  403)
        return

    def test_default_user(self):
        self.login_user("other_user")
        # try each function, test result
        self.assertEqual(self.view_site_home_page().status_code,    200)
        self.assertEqual(self.create_collection().status_code,      403)
        self.assertEqual(self.remove_collection().status_code,      403)
        #
        self.assertEqual(self.list_users().status_code,             403)
        self.assertEqual(self.view_user_get().status_code,          403)
        self.assertEqual(self.view_user_edit().status_code,         403)
        self.assertEqual(self.view_user_copy().status_code,         403)
        self.assertEqual(self.new_user().status_code,               403)
        self.assertEqual(self.copy_user().status_code,              403)
        self.assertEqual(self.edit_user().status_code,              403)
        self.assertEqual(self.delete_user().status_code,            403)
        self.assertEqual(self.delete_user_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_types().status_code,             200)
        self.assertEqual(self.view_type_get().status_code,          200)
        self.assertEqual(self.view_type_edit().status_code,         403)
        self.assertEqual(self.view_type_copy().status_code,         403)
        self.assertEqual(self.new_type().status_code,               403)
        self.assertEqual(self.copy_type().status_code,              403)
        self.assertEqual(self.edit_type().status_code,              403)
        self.assertEqual(self.delete_type().status_code,            403)
        self.assertEqual(self.delete_type_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_data().status_code,              200)
        self.assertEqual(self.view_data_get().status_code,          200)
        self.assertEqual(self.view_data_edit().status_code,         403)
        self.assertEqual(self.view_data_copy().status_code,         403)
        self.assertEqual(self.new_data().status_code,               403)
        self.assertEqual(self.copy_data().status_code,              403)
        self.assertEqual(self.edit_data().status_code,              403)
        self.assertEqual(self.delete_data().status_code,            403)
        self.assertEqual(self.delete_data_confirmed().status_code,  403)
        return

    def test_no_login_user(self):
        # try each function, test result
        self.assertEqual(self.view_site_home_page().status_code,    200)
        self.assertEqual(self.create_collection().status_code,      401)
        self.assertEqual(self.remove_collection().status_code,      401)
        #
        self.assertEqual(self.list_users().status_code,             401)
        self.assertEqual(self.view_user_get().status_code,          401)
        self.assertEqual(self.view_user_edit().status_code,         401)
        self.assertEqual(self.view_user_copy().status_code,         401)
        self.assertEqual(self.new_user().status_code,               401)
        self.assertEqual(self.copy_user().status_code,              401)
        self.assertEqual(self.edit_user().status_code,              401)
        self.assertEqual(self.delete_user().status_code,            401)
        self.assertEqual(self.delete_user_confirmed().status_code,  401)
        #
        self.assertEqual(self.list_types().status_code,             200)
        self.assertEqual(self.view_type_get().status_code,          200)
        self.assertEqual(self.view_type_edit().status_code,         401)
        self.assertEqual(self.view_type_copy().status_code,         401)
        self.assertEqual(self.new_type().status_code,               401)
        self.assertEqual(self.copy_type().status_code,              401)
        self.assertEqual(self.edit_type().status_code,              401)
        self.assertEqual(self.delete_type().status_code,            401)
        self.assertEqual(self.delete_type_confirmed().status_code,  401)
        #
        self.assertEqual(self.list_data().status_code,              200)
        self.assertEqual(self.view_data_get().status_code,          200)
        self.assertEqual(self.view_data_edit().status_code,         401)
        self.assertEqual(self.view_data_copy().status_code,         401)
        self.assertEqual(self.new_data().status_code,               401)
        self.assertEqual(self.copy_data().status_code,              401)
        self.assertEqual(self.edit_data().status_code,              401)
        self.assertEqual(self.delete_data().status_code,            401)
        self.assertEqual(self.delete_data_confirmed().status_code,  401)
        return

    # Test site-level permissions

    def test_site_admin_user(self):
        self.login_user("user_site_admin")
        # try each function, test result
        self.assertEqual(self.create_collection().status_code,      302)
        self.assertEqual(self.remove_collection().status_code,      200)
        #
        self.assertEqual(self.list_users().status_code,             200)
        self.assertEqual(self.view_user_get().status_code,          200)
        self.assertEqual(self.view_user_edit().status_code,         302)
        self.assertEqual(self.view_user_copy().status_code,         302)
        self.assertEqual(self.new_user().status_code,               302)
        self.assertEqual(self.copy_user().status_code,              302)
        self.assertEqual(self.edit_user().status_code,              302)
        self.assertEqual(self.delete_user().status_code,            200)
        self.assertEqual(self.delete_user_confirmed().status_code,  302)
        #
        self.assertEqual(self.list_types().status_code,             200)
        self.assertEqual(self.view_type_get().status_code,          200)
        self.assertEqual(self.view_type_edit().status_code,         302)
        self.assertEqual(self.view_type_copy().status_code,         302)
        self.assertEqual(self.new_type().status_code,               302)
        self.assertEqual(self.copy_type().status_code,              302)
        self.assertEqual(self.edit_type().status_code,              302)
        self.assertEqual(self.delete_type().status_code,            200)
        self.assertEqual(self.delete_type_confirmed().status_code,  302)
        #
        self.assertEqual(self.list_data().status_code,              200)
        self.assertEqual(self.view_data_get().status_code,          200)
        self.assertEqual(self.view_data_edit().status_code,         302)
        self.assertEqual(self.view_data_copy().status_code,         302)
        self.assertEqual(self.new_data().status_code,               302)
        self.assertEqual(self.copy_data().status_code,              302)
        self.assertEqual(self.edit_data().status_code,              302)
        self.assertEqual(self.delete_data().status_code,            200)
        self.assertEqual(self.delete_data_confirmed().status_code,  302)
        return

    def test_site_create_user(self):
        self.login_user("user_site_create_coll")
        # try each function, test result
        self.assertEqual(self.create_collection().status_code,      302)
        self.assertEqual(self.remove_collection().status_code,      403)
        #
        self.assertEqual(self.list_users().status_code,             403)
        self.assertEqual(self.view_user_get().status_code,          403)
        self.assertEqual(self.view_user_edit().status_code,         403)
        self.assertEqual(self.view_user_copy().status_code,         403)
        self.assertEqual(self.new_user().status_code,               403)
        self.assertEqual(self.copy_user().status_code,              403)
        self.assertEqual(self.edit_user().status_code,              403)
        self.assertEqual(self.delete_user().status_code,            403)
        self.assertEqual(self.delete_user_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_types().status_code,             200)
        self.assertEqual(self.view_type_get().status_code,          200)
        self.assertEqual(self.view_type_edit().status_code,         403)
        self.assertEqual(self.view_type_copy().status_code,         403)
        self.assertEqual(self.new_type().status_code,               403)
        self.assertEqual(self.copy_type().status_code,              403)
        self.assertEqual(self.edit_type().status_code,              403)
        self.assertEqual(self.delete_type().status_code,            403)
        self.assertEqual(self.delete_type_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_data().status_code,              200)
        self.assertEqual(self.view_data_get().status_code,          200)
        self.assertEqual(self.view_data_edit().status_code,         403)
        self.assertEqual(self.view_data_copy().status_code,         403)
        self.assertEqual(self.new_data().status_code,               403)
        self.assertEqual(self.copy_data().status_code,              403)
        self.assertEqual(self.edit_data().status_code,              403)
        self.assertEqual(self.delete_data().status_code,            403)
        self.assertEqual(self.delete_data_confirmed().status_code,  403)
        return

    def test_site_delete_user(self):
        self.login_user("user_site_delete_coll")
        # try each function, test result
        self.assertEqual(self.create_collection().status_code,      302)
        self.assertEqual(self.remove_collection().status_code,      200)
        #
        self.assertEqual(self.list_users().status_code,             403)
        self.assertEqual(self.view_user_get().status_code,          403)
        self.assertEqual(self.view_user_edit().status_code,         403)
        self.assertEqual(self.new_user().status_code,               403)
        self.assertEqual(self.copy_user().status_code,              403)
        self.assertEqual(self.edit_user().status_code,              403)
        self.assertEqual(self.delete_user().status_code,            403)
        self.assertEqual(self.delete_user_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_types().status_code,             200)
        self.assertEqual(self.view_type_get().status_code,          200)
        self.assertEqual(self.view_type_edit().status_code,         403)
        self.assertEqual(self.view_type_copy().status_code,         403)
        self.assertEqual(self.new_type().status_code,               403)
        self.assertEqual(self.copy_type().status_code,              403)
        self.assertEqual(self.edit_type().status_code,              403)
        self.assertEqual(self.delete_type().status_code,            403)
        self.assertEqual(self.delete_type_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_data().status_code,              200)
        self.assertEqual(self.view_data_get().status_code,          200)
        self.assertEqual(self.view_data_edit().status_code,         403)
        self.assertEqual(self.view_data_copy().status_code,         403)
        self.assertEqual(self.new_data().status_code,               403)
        self.assertEqual(self.copy_data().status_code,              403)
        self.assertEqual(self.edit_data().status_code,              403)
        self.assertEqual(self.delete_data().status_code,            403)
        self.assertEqual(self.delete_data_confirmed().status_code,  403)
        return

    def test_site_view_user(self):
        self.login_user("user_site_view")
        # try each function, test result
        self.assertEqual(self.create_collection().status_code,      403)
        self.assertEqual(self.remove_collection().status_code,      403)
        #
        self.assertEqual(self.list_users().status_code,             403)
        self.assertEqual(self.view_user_get().status_code,          403)
        self.assertEqual(self.view_user_edit().status_code,         403)
        self.assertEqual(self.view_user_copy().status_code,         403)
        self.assertEqual(self.new_user().status_code,               403)
        self.assertEqual(self.copy_user().status_code,              403)
        self.assertEqual(self.edit_user().status_code,              403)
        self.assertEqual(self.delete_user().status_code,            403)
        self.assertEqual(self.delete_user_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_types().status_code,             200)
        self.assertEqual(self.view_type_get().status_code,          200)
        self.assertEqual(self.view_type_edit().status_code,         403)
        self.assertEqual(self.view_type_copy().status_code,         403)
        self.assertEqual(self.new_type().status_code,               403)
        self.assertEqual(self.copy_type().status_code,              403)
        self.assertEqual(self.edit_type().status_code,              403)
        self.assertEqual(self.delete_type().status_code,            403)
        self.assertEqual(self.delete_type_confirmed().status_code,  403)
        #
        self.assertEqual(self.list_data().status_code,              200)
        self.assertEqual(self.view_data_get().status_code,          200)
        self.assertEqual(self.view_data_edit().status_code,         403)
        self.assertEqual(self.view_data_copy().status_code,         403)
        self.assertEqual(self.new_data().status_code,               403)
        self.assertEqual(self.copy_data().status_code,              403)
        self.assertEqual(self.edit_data().status_code,              403)
        self.assertEqual(self.delete_data().status_code,            403)
        self.assertEqual(self.delete_data_confirmed().status_code,  403)
        return

    # Test permissions effect on entity lists

    def test_list_all(self):
        # Only list things we can view/list
        u = entitydata_list_all_url("testcoll", list_id="Default_list_all")
        r = self.client.get(u)
        self.assertEqual(r.status_code,   200)
        self.assertEqual(r.reason_phrase, "OK")
        # Test context
        # Entities and bound fields
        entities = context_list_entities(r.context)
        self.assertEqual(len(entities), 2)
        entity_fields = (
            [ ('testtype',    '_type',    'RecordType testcoll/testtype')
            , ('entity1',     'testtype', 'Entity testcoll/testtype/entity1')
            ])
        for eid in range(2):
            for fid in range(3):
                item_field_value = context_list_item_field_value(r.context, entities[eid], fid)
                self.assertEqual(item_field_value, entity_fields[eid][fid])
        return

# End.



# Test settings
#
# A copy of initial data is created within the project
# Service configuration is kept under personal home directory to protect secret keys, etc
#

import os

from common import *

import logging
log = logging.getLogger(__name__)

ANNALIST_VERSION_MSG = "Annalist version %s (test configuration)"%(ANNALIST_VERSION)

# Override authentication backend to use local database only
AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
    )

# Override logging setings for tests
LOGGING_FILE = SITE_SRC_ROOT+'/annalist.log'
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        'timed': {
            'format': '%(levelname)s %(asctime)s %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse',
        },
        'require_debug_true': {
            '()': 'django.utils.log.RequireDebugTrue',
        },
    },
    'handlers': {
        'console': {
            'level': 'WARNING',
            'class': 'logging.StreamHandler',
            'formatter': 'simple',
        },
        'logfile': {
            'level': 'DEBUG',
            'class': 'logging.handlers.WatchedFileHandler',
            'filename': LOGGING_FILE,
            'formatter': 'timed'
        },
        'null': {
            'class': 'logging.NullHandler',
        }
    },
    'loggers': {
        # '': {
        #      'handlers': ['console'],
        #      'level': 'INFO',
        #  },
        'annalist': {
            'handlers': ['console', 'logfile'],
            'level': 'DEBUG',
            'propagate': False
        },
        'django': {
            'handlers': ['console', 'logfile'],
        },
        'py.warnings': {
            'handlers': ['console', 'logfile'],
        },
    }
}

# URI parts used for testing
TEST_HOST           = "test.example.com"
TEST_HOST_URI       = "http://"+TEST_HOST
TEST_BASE_PATH      = "testsite"
TEST_BASE_URI       = TEST_HOST_URI+"/"+TEST_BASE_PATH

# Doesn't seem to be working...
TEST_DISCOVER_TOP_LEVEL = os.path.join(SITE_SRC_ROOT, "annalist")
os.chdir(TEST_DISCOVER_TOP_LEVEL)

# Logging level used by selected log statements whose output may be useful
# for tracing field values displayed in Annalist edit/view forms.
# Suggested use is to raise level to logging.INFO when running a single named
# test, when trying to understand how values end up in a form.
TRACE_FIELD_VALUE   = logging.DEBUG

# Override root URI configuration for tests
ROOT_URLCONF        = 'annalist_site.runtests_urls'

SETTINGS_MODULE     = __name__
BASE_DATA_DIR       = SITE_SRC_ROOT+"/sampledata/data"
BASE_SITE_DIR       = os.path.join(BASE_DATA_DIR, layout.SITE_DIR)
CONFIG_BASE         = os.path.join(os.path.expanduser("~"), ".annalist/")

log.info(ANNALIST_VERSION_MSG)
# log.info("Annalist version %s (test configuration)"%(ANNALIST_VERSION))
log.info("SETTINGS_MODULE: "+SETTINGS_MODULE)
log.info("BASE_DATA_DIR:   "+BASE_DATA_DIR)
log.info("CONFIG_BASE:     "+CONFIG_BASE)
log.info("DJANGO_ROOT:     "+DJANGO_ROOT)
log.info("SITE_CONFIG_DIR: "+SITE_CONFIG_DIR)
log.info("SITE_SRC_ROOT:   "+SITE_SRC_ROOT)
log.info("DB PATH:         "+DATABASES['default']['NAME'])
# log.info("TEST_RUNNER:             "+TEST_RUNNER)
# log.info("TEST_DISCOVER_TOP_LEVEL: "+TEST_DISCOVER_TOP_LEVEL)
log.debug("Test debug log")

# End.



"""
Annalist application hooks
(cf. https://docs.djangoproject.com/en/dev/ref/applications/)

The main purpose of this is to log settings values to the loig file, 
after the log file configuration has been applied.
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

from django.conf import settings
from django.apps import AppConfig

import logging
log = logging.getLogger(__name__)

class AnnalistConfig(AppConfig):
    name = 'annalist'

    def ready(self):
        log.info(settings.ANNALIST_VERSION_MSG)
        log.info("SETTINGS_MODULE:  "+settings.SETTINGS_MODULE)
        log.info("BASE_DATA_DIR:    "+settings.BASE_DATA_DIR)
        log.info("CONFIG_BASE:      "+settings.CONFIG_BASE)
        log.info("DJANGO_ROOT:      "+settings.DJANGO_ROOT)
        log.info("SITE_CONFIG_DIR:  "+settings.SITE_CONFIG_DIR)
        log.info("SITE_SRC_ROOT:    "+settings.SITE_SRC_ROOT)
        log.info("STATICFILES_DIRS: "+settings.STATICFILES_DIRS[1])
        log.info("DB PATH:          "+settings.DATABASES['default']['NAME'])
        log.info("ALLOWED_HOSTS:    "+",".join(settings.ALLOWED_HOSTS))
        log.info("LOGGING_FILE:     "+settings.LOGGING_FILE)
        return

# End.



# ro_uriutils.py

"""
Helper functions for manipulasting and testing URIs and URI-related file paths,
and for accessing or testing data at a URI reference.
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2011-2013, University of Oxford"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import sys
import os
import os.path
import re
import urllib
import urlparse
import httplib

import logging
log = logging.getLogger(__name__)

fileuribase = "file://"

def isFileUri(uri):
    return uri.startswith(fileuribase)

def resolveUri(uriref, base, path=""):
    """
    Resolve a URI reference against a supplied base URI and path (supplied as strings).
    (The path is a local file system path, and may need converting to use URI conventions)
    """
    upath = urllib.pathname2url(path)
    if os.path.isdir(path) and not upath.endswith('/'):
        upath = upath + '/'
    return urlparse.urljoin(urlparse.urljoin(base, upath), uriref)

def resolveFileAsUri(path):
    """
    Resolve a filename reference against the current working directory, and return the
    corresponding file:// URI.
    
    If the supplied string is already a URI, it is returned unchanged
    (for idempotency and non-file URIs)
    """
    if urlparse.urlsplit(path).scheme == "":
        path = resolveUri("", fileuribase, os.path.abspath(path))
    return path

def getFilenameFromUri(uri):
    """
    Convert a file:// URI into a local file system reference
    """
    uriparts = urlparse.urlsplit(uri)
    assert uriparts.scheme == "file", "RO %s is not in local file system"%uri
    uriparts = urlparse.SplitResult("","",uriparts.path,uriparts.query,uriparts.fragment)
    return urllib.url2pathname(urlparse.urlunsplit(uriparts))

def isLiveUri(uriref):
    """
    Test URI reference to see if it refers to an accessible resource
    
    Relative URI references are assumed to be local file system references,
    relartive to the current working directory.
    """
    islive  = False
    fileuri = resolveFileAsUri(uriref)
    if isFileUri(fileuri):
        islive = os.path.exists(getFilenameFromUri(fileuri))
    else:
        parseduri = urlparse.urlsplit(uriref)
        scheme    = parseduri.scheme
        host      = parseduri.netloc
        path      = parseduri.path
        if parseduri.query: path += "?"+parseduri.query
        httpcon   = httplib.HTTPConnection(host, timeout=5)
        # Extra request headers
        # ... none for now
        # Execute request
        try:
            httpcon.request("HEAD", path)
            response = httpcon.getresponse()
            status   = response.status
        except:
            status   = 900
        # Pick out elements of response
        islive = (status >= 200) and (status <= 299)
    return islive

def retrieveUri(uriref):
    # @@TODO: revise to use httplib2, or delete this method
    uri = resolveUri(uriref, fileuribase, os.getcwd())
    request  = urllib2.Request(uri)
    try:
        response = urllib2.urlopen(request)
        result   = response.read()
    except:
        result = None
    return result

# End.



"""
Collection of Annalist data records for a specified record type
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import os
import os.path
import urlparse
import shutil

import logging
log = logging.getLogger(__name__)

from django.conf import settings

from annalist                   import layout
from annalist.exceptions        import Annalist_Error
from annalist.identifiers       import ANNAL
from annalist                   import util
from annalist.models.entity     import Entity

class EntityData(Entity):

    _entitytype     = ANNAL.CURIE.EntityData
    _entitytypeid   = None
    _entityview     = layout.TYPEDATA_ENTITY_VIEW
    _entitypath     = layout.TYPEDATA_ENTITY_PATH
    _entityfile     = layout.ENTITY_DATA_FILE
    _entityref      = layout.DATA_ENTITY_REF
    _contextref     = layout.ENTITY_CONTEXT_FILE

    def __init__(self, parent, entity_id, altparent=None, use_altpath=False):
        """
        Initialize a new Entity Data object, without metadata.

        parent      is the parent collection (RecordType) from which the entity is descended.
        entity_id   the local identifier (slug) for the data record
        altparent   is an alternative parent entity to search for this entity, using 
                    the alternative path for the entity type: this is used to augment 
                    explicitly created entities in a collection with site-wide 
                    installed metadata entites (i.e. types, views, etc.)
        use_altpath is set True if this entity is situated at the alternative
                    path relative to its parent.
        """
        super(EntityData, self).__init__(parent, entity_id, altparent=altparent, use_altpath=use_altpath)
        self._entitytypeid  = self._entitytypeid or parent.get_id()
        self._paramdict     = { 'type_id': self._entitytypeid, 'id': entity_id }
        self._entityviewuri = parent._entityurl+self._entityview%self._paramdict
        # self._entityref     = layout.CONTEXT_ENTITY_REF%self._paramdict
        log.debug("EntityData: _entityviewuri %s"%(self._entityviewuri))
        return

    def _migrate_filenames(self):
        """
        Default method for filename migration.

        Returns a list of filenames used for the current entity type in previous
        versions of Annalist software.  If the expected filename is not found when 
        attempting to read a file, the _load_values() method calls this function to
        and looks for any of the filenames returned.  If found, the file is renamed
        to the current version filename.

        Default method returns an empty list.
        """
        return [layout.ENTITY_OLD_DATA_FILE]

# End.



# Root settings module
#
# Default submodule is selected by manage.py
#
# To specify an alternative:
#   python manage.py runserver --settings=annalist_site.settings.personal
# or
#   export DJANGO_SETTINGS_MODULE=annalist_site.settings.personal
#
# See also:
#   http://www.rdegges.com/the-perfect-django-settings-file/
#

import sys

import logging
logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)
log.debug("Python path:    "+repr(sys.path))

# End.



"""
Tests for text value displayed as a hyperlink.

The text value is taken to be a URI or URI reference.

@@TODO: later developments shoukld also support CURIES (cf. issue #19,
https://github.com/gklyne/annalist/issues/19).
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import sys
import os
import unittest
import re
from collections import OrderedDict

import logging
log = logging.getLogger(__name__)

# from django.conf                                import settings
# from django.test                                import TestCase # cf. https://docs.djangoproject.com/en/dev/topics/testing/tools/#assertions
# from django.template                            import Context, Template, loader

from annalist.views.fields.render_uri_link import (
    get_uri_link_renderer, 
    URILinkValueMapper
    )

from annalist.tests.field_rendering_support     import FieldRendererTestSupport

class UriLinkRenderingTest(FieldRendererTestSupport):

    def setUp(self):
        return

    def tearDown(self):
        return

    def test_RenderUriLinkValue(self):

        def expect_render(linktext, labeltext):
            render_view = '''<a href="%s" target="_blank">%s</a>'''%(linktext, labeltext)
            render_edit = (
                '''<input type="text" size="64" name="repeat_prefix_test_field" '''+
                       '''placeholder="(test placeholder)" '''+
                       '''value="%s" />'''
                )%linktext
            return {'view': render_view, 'edit': render_edit}

        test_values = (
            [ ("http://example.com/path",         "example.com/path")
            , ("https://example.com/path",        "example.com/path")
            , ("file:///example/path",            "example/path")
            , ("file://example.com/path",         "example.com/path")
            , ("mailto:user@example.com",         "user@example.com")
            , ("foo://example.com/more",          "foo://example.com/more")
            ])
        test_value_context_renders = (
            [ (self._make_test_context(linktext),  expect_render(linktext, labeltext))
                for linktext, labeltext in test_values
            ])
        renderer = get_uri_link_renderer()

        for render_context, expect_render in test_value_context_renders:
            # print repr(render_context['field']['field_value'])
            self._check_value_renderer_results(
                renderer,
                context=render_context,
                expect_rendered_view=expect_render['view'],
                expect_rendered_edit=expect_render['edit'],
                collapse_whitespace=True
                )
        return

    def test_DecodeUriLinkValue(self):
        test_decode_values = (
            { None:                         ""
            , "http://example.com/path":    "http://example.com/path"
            , "https://example.com/path":   "https://example.com/path"
            , "example.com/path":           "example.com/path"
            , "file:///example/path/":      "file:///example/path/"
            , "file://example.com/path":    "file://example.com/path"
            , "mailto:user@example.com":    "mailto:user@example.com"
            , "foo://example.com/more":     "foo://example.com/more"
            })
        for valtext, expect_valdata in test_decode_values.items():
            valdata = URILinkValueMapper.decode(valtext)
            self.assertEqual(
                valdata, expect_valdata, 
                "Value decode(%s) = %r, expect %r"%(valtext, valdata, expect_valdata)
                )
        return

# End.

if __name__ == "__main__":
    # import django
    # django.setup()  # Needed for template loader
    # Runtests in this module
    # runner = unittest.TextTestRunner(verbosity=2)
    # tests = unittest.TestSuite()
    # tests  = getSuite(select=sel)
    # if tests: runner.run(tests)
    unittest.main()



# !/usr/bin/env python
#
# am_main.py - command line tool to perform Annalist installation management
#

from __future__ import print_function

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2013-2014, Graham Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import sys
import os
import os.path
import re
import argparse
import logging
import errno

log = logging.getLogger(__name__)

# if __name__ == "__main__":
dirhere = os.path.dirname(os.path.realpath(__file__))
annroot = os.path.dirname(os.path.join(dirhere))
sys.path.insert(0, annroot)
# sys.path.insert(0, dirhere)

import annalist

# from annalist_manager       import am_errors
import am_errors
from am_runtests            import am_runtests
from am_runserver           import am_runserver, am_serverlog, am_sitedirectory, am_version
from am_initialize          import am_initialize
from am_createuser          import (
    am_createadminuser, am_defaultadminuser, am_updateadminuser, 
    am_setdefaultpermissions, am_setpublicpermissions,
    am_deleteuser
    )
from am_createsite          import am_createsite, am_updatesite
from am_help                import am_help, command_summary_help

VERSION = annalist.__version__

def progname(args):
    return os.path.basename(args[0])

def parseCommandArgs(argv):
    """
    Parse command line arguments

    argv            argument list from command line

    Returns a pair consisting of options specified as returned by
    OptionParser, and any remaining unparsed arguments.
    """
    # create a parser for the command line options
    parser = argparse.ArgumentParser(
                description="Annalist site management utility",
                formatter_class=argparse.RawDescriptionHelpFormatter,
                epilog=command_summary_help
                )
    parser.add_argument('--version', action='version', version='%(prog)s '+VERSION)
    parser.add_argument("-c", "--configuration",
                        action='store',
                        dest="configuration", metavar="CONFIG",
                        default="personal",
                        #choices=['personal', 'shared', 'devel', 'runtests'],
                        help="Select site configuration by name (e.g. personal, shared, devel, runtests.")
    parser.add_argument("-p", "--personal",
                        action='store_true',
                        dest="config_p", # metavar="PERSONAL",
                        help="Select personal site configuration.")
    parser.add_argument("-d", "--development",
                        action='store_true',
                        dest="config_d", # metavar="DEVELOPMENT",
                        help="Select development site configuration.")
    parser.add_argument("-s", "--shared",
                        action='store_true',
                        dest="config_s", # metavar="SHARED",
                        help="Select shared site configuration.")
    parser.add_argument("-f", "--force",
                        action='store_true',
                        dest="force",
                        help="Force overwrite of existing site data.")
    parser.add_argument("--debug",
                        action="store_true", 
                        dest="debug", 
                        default=False,
                        help="Run with full debug output enabled")
    parser.add_argument("command", metavar="COMMAND",
                        nargs=None,
                        help="sub-command, one of the options listed below."
                        )
    parser.add_argument("args", metavar="ARGS",
                        nargs="*",
                        help="Additional arguments, depending on the command used."
                        )
    # parse command line now
    options = parser.parse_args(argv)
    if options:
        if options.config_d: options.configuration = "devel"
        if options.config_p: options.configuration = "personal"
        if options.config_s: options.configuration = "shared"
        if options and options.command:
            return options
    print("No valid usage option given.", file=sys.stderr)
    parser.print_usage()
    return None

def run(userhome, userconfig, options, progname):
    """
    Command line tool to create and submit deposit information packages
    """
    if options.command.startswith("runt"):                  # runtests
        return am_runtests(annroot, options)
    if options.command.startswith("init"):                  # initialize (intsllaation, django database)
        return am_initialize(annroot, userhome, userconfig, options)
    if options.command.startswith("createa"):               # createadminuser
        return am_createadminuser(annroot, userhome, options)
    if options.command.startswith("defaulta"):              # defaultadminuser
        return am_defaultadminuser(annroot, userhome, options)
    if options.command.startswith("updatea"):               # updateadminuser
        return am_updateadminuser(annroot, userhome, options)
    if options.command.startswith("setdef"):                # setdefaultpermissions
        return am_setdefaultpermissions(annroot, userhome, options)
    if options.command.startswith("setpub"):                # setpublicpermissions
        return am_setpublicpermissions(annroot, userhome, options)
    if options.command.startswith("deleteu"):               # deleteuser
        return am_deleteuser(annroot, userhome, options)
    if options.command.startswith("creates"):               # createsitedata
        return am_createsite(annroot, userhome, options)
    if options.command.startswith("updates"):               # updatesitedata
        return am_updatesite(annroot, userhome, options)
    if options.command.startswith("runs"):                  # runserver
        return am_runserver(annroot, userhome, options)
    if options.command.startswith("serv"):                  # serverlog
        return am_serverlog(annroot, userhome, options)
    if options.command.startswith("site"):                  # sitedir
        return am_sitedirectory(annroot, userhome, options)
    if options.command.startswith("ver"):                   # version
        return am_version(annroot, userhome, options)
    if options.command.startswith("help"):
        return am_help(options, progname)
    print("Un-recognised sub-command: %s"%(options.command), file=sys.stderr)
    print("Use '%s --help' to see usage summary"%(progname), file=sys.stderr)        
    return am_errors.AM_BADCMD

def runCommand(userhome, userconfig, argv):
    """
    Run program with supplied configuration base directory, Base directory
    from which to start looking for research objects, and arguments.

    This is called by main function (below), and also by test suite routines.

    Returns exit status.
    """
    options = parseCommandArgs(argv[1:])
    if options and options.debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)
    # log.debug("runCommand: configbase %s, filebase %s, argv %s"%(configbase, filebase, repr(argv)))
    # log.debug("Options: %s"%(repr(options)))
    # else:
    #     logging.basicConfig()
    if options:
        progname = os.path.basename(argv[0])
        status   = run(userhome, userconfig, options, progname)
    else:
        status = am_errors.AM_BADCMD
    return status

def runMain():
    """
    Main program transfer function for setup.py console script
    """
    userhome   = os.path.expanduser("~")
    userconfig = os.path.join(userhome, ".annalist")
    return runCommand(userhome, userconfig, sys.argv)

if __name__ == "__main__":
    """
    Program invoked from the command line.
    """
    p = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    sys.path.insert(0, p)
    status = runMain()
    sys.exit(status)

# Tests
#
# python am_main.py runtests
# python am_main.py --config=runtests initialize
# python am_main.py --config=runtests createadminuser testtestadmin testestadmin@localhost
# python am_main.py --config=runtests createadminuser testtestadmin testestadmin@localhost
# python am_main.py --config=runtests deleteuser testtestadmin
# python am_main.py --config=runtests deleteuser testtestadmin
# python am_main.py --config=runtests updateadminuser gklyne
# python am_main.py --config=runtests setdefaultpermissions "VIEW CREATE TEST"
# python am_main.py --config=runtests setpublicpermissions "VIEW TEST"
# python am_main.py --config=runtests updatesitedata

# End.




"""
Common base classes for Annalist stored entities (collections, data, metadata, etc.)

This module implements a common pattern whereby an entity is related to a parent,
with storage directories and URIs allocated by combining the parent entity and a
local identifier (slug) for the descendent.

Part of the purpose of this module is to abstract the underlying storage access
from the Annalist organization of presented entities.
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import os
import os.path
import urlparse
import shutil
import json
import errno

import logging
log = logging.getLogger(__name__)

from django.conf                import settings

from annalist                   import util
from annalist.exceptions        import Annalist_Error
from annalist.identifiers       import ANNAL

from annalist.models.entityroot import EntityRoot

#   -------------------------------------------------------------------------------------------
#
#   Entity
#
#   -------------------------------------------------------------------------------------------

class Entity(EntityRoot):
    """
    This is the base class for all entities managed by Annalist as 
    descendents of some other entity.
    """

    _entitytype     = ANNAL.CURIE.Entity
    _entitytypeid   = None
    _entityview     = "%(id)s/"     # Placeholder for testing
    _entitypath     = None          # Relative path from parent to entity (template)
    _entityfile     = None          # Relative reference to body file from entity
    _entityref      = None          # Relative reference to entity from body file
    _last_id        = None          # Last ID allocated

    def __init__(self, parent, entityid, altparent=None, idcheck=True, use_altpath=False):
        """
        Initialize a new Entity object, possibly without values.  The created
        entity is not saved to disk at this stage - see ._save() method.

        parent      is the parent entity from which the new entity is descended.
        entityid    the collection identifier for the collection
        altparent   is an alternative parent entity to search for this entity, using 
                    the alternative path for the entity type: this is used to augment 
                    explicitly created entities in a collection with site-wide 
                    installed metadata entites (i.e. types, views, etc.)
        idcheck     is set False to skip the valid-identifier check
        use_altpath is set True if this entity is situated at the alternative
                    path relative to its parent.  This is used by test code to force
                    an entity to be located on the alternative parent path.  In normal
                    use, this value is left unspecified (i.e. False).
                    @@TODO: when collection-inheritance is introduced, this should go.
        """
        if idcheck and not util.valid_id(entityid):
            raise ValueError("Invalid entity identifier: %s"%(entityid))
        relpath = self.relpath(entityid)
        if use_altpath:
            relpath = self.altpath(entityid)
        super(Entity, self).__init__(parent._entityurl+relpath, parent._entitydir+relpath)
        self._entityviewuri = parent._entityurl+self._entityview%{'id': entityid, 'type_id': self._entitytypeid}
        self._entityalturi  = None
        self._entityaltdir  = None
        if altparent:
            altpath = self.altpath(entityid)
            self._entityalturi = altparent._entityurl+altpath
            self._entityaltdir = altparent._entitydir+altpath
            self._entityuseuri = None   # URI not known until entity is created or accessed
            log.debug("Entity.__init__: entity alt URI %s, entity alt dir %s"%(self._entityalturi, self._entityaltdir))
        self._entityid = entityid
        log.debug("Entity.__init__: entity_id %s, type_id %s"%(self._entityid, self.get_type_id()))
        return

    def get_view_url(self, baseurl=""):
        """
        Return URI used to view entity data.  For metadata entities, this may be 
        different from the URI at which the resource is located, per get_uri().
        The intent is to provide a URI that works regardless of whether the metadata
        is stored as site-wide or collection-specific data.
        """
        return urlparse.urljoin(baseurl, self._entityviewuri)

    # I/O helper functions

    @classmethod
    def allocate_new_id(cls, parent):
        if cls._last_id is None:
            cls._last_id = 1
        while True:
            newid = "%08d"%cls._last_id
            if not cls.exists(parent, newid):
                break
            cls._last_id += 1
        return newid

    @classmethod
    def relpath(cls, entityid):
        """
        Returns parent-relative path string for an identified entity of the given class.

        cls         is the class of the entity whose relative path is returned.
        entityid    is the local identifier (slug) for the entity.
        """
        log.debug("Entity.relpath: entitytype %s, entityid %s"%(cls._entitytype, entityid))
        relpath = (cls._entitypath or "%(id)s")%{'id': entityid, 'type_id': cls._entitytypeid}
        log.debug("Entity.relpath: %s"%(relpath))
        return relpath

    @classmethod
    def altpath(cls, entityid):
        """
        Returns alternative-parent-relative path string for an identified entity 
        of the given class.

        cls         is the class of the entity whose alternative relative path is returned.
        entityid    is the local identifier (slug) for the entity.
        """
        log.debug("Entity.altpath: entitytype %s, entityid %s"%(cls._entitytype, entityid))
        altpath = (cls._entityaltpath or "%(id)s")%{'id': entityid, 'type_id': cls._entitytypeid}
        log.debug("Entity.altpath: %s"%(altpath))
        return altpath

    @classmethod
    def path(cls, parent, entityid):
        """
        Returns path string for accessing the body of the indicated entity.

        cls         is the class of the entity whose path is returned.
        parent      is the parent from which the entity is descended.
        entityid    is the local identifier (slug) for the entity.
        """
        log.debug("Entity.path: entitytype %s, parentdir %s, entityid %s"%
            (cls._entitytype, parent._entitydir, entityid)
            )
        assert cls._entityfile is not None
        p = util.entity_path(parent._entitydir, [cls.relpath(entityid)], cls._entityfile)
        log.debug("Entity.path: %s"%(p))
        return p

    # Create and access functions

    def child_entity_ids(self, cls, altparent=None):
        """
        Iterates over child entity identifiers of an indicated class.
        The supplied class is used to determine a subdirectory to be scanned.

        cls         is a subclass of Entity indicating the type of children to
                    iterate over.
        altparent   is an alternative parent entity to be checked using the class's
                    alternate relative path, or None if only potential child IDs of the
                    current entity are returned.
        """
        for i in self._children(cls, altparent=altparent):
            if cls.exists(self, i, altparent=altparent):
                yield i
        return

    def child_entities(self, cls, altparent=None):
        """
        Iterates over child entities of an indicated class.
        The supplied class is used to determine a subdirectory to be scanned, 
        and to instantiate and load data for the entities found.

        cls         is a subclass of Entity indicating the type of children to
                    iterate over.
        altparent   is an alternative parent entity to be checked using the class's
                    alternate relative path, or None if only potential child IDs of the
                    current entity are returned.
        """
        for i in self._children(cls, altparent=altparent):
            e = cls.load(self, i, altparent=altparent)
            if e:
                yield e
        return

    @classmethod
    def _child_init(cls, parent, entityid, altparent=None, use_altpath=False):
        """
        Instantiate a child entity (e.g. for create and load methods)
        """
        if use_altpath:
            e = cls(parent, entityid, use_altpath=use_altpath)
        elif altparent:
            e = cls(parent, entityid, altparent=altparent)
        else:
            e = cls(parent, entityid)
        return e

    @classmethod
    def create(cls, parent, entityid, entitybody, use_altpath=False):
        """
        Method creates a new entity or rewrites an existing entity.

        cls         is a class value used to construct the new entity value
        parent      is the parent entity from which the new entity is descended.
        entityid    is the local identifier (slug) for the new entity - this is 
                    required to be unique among descendents of a common parent.
        entitybody  is a dictionary of values that are stored for the created entity.
        use_altpath is set True if the entity is to be situated at the alternative
                    path relative to its parent.

        Returns the created entity as an instance of the supplied class object.
        """
        log.debug("Entity.create: entityid %s"%(entityid))
        e = cls._child_init(parent, entityid, use_altpath=use_altpath)
        e.set_values(entitybody)
        e._save()
        return e

    @classmethod
    def load(cls, parent, entityid, altparent=None, use_altpath=False):
        """
        Return an entity with given identifier belonging to some given parent,
        or None if there is not such identity.

        cls         is the class of the entity to be loaded
        parent      is the parent from which the entity is descended.
        entityid    is the local identifier (slug) for the entity.
        altparent   is an alternative parent entity to search for the loaded entity, 
                    using the alternative path for the entity type.
        use_altpath is set True if this entity is situated at the alternative
                    path relative to its parent.

        Returns an instance of the indicated class with data loaded from the
        corresponding Annalist storage, or None if there is no such entity.
        """
        log.debug("Entity.load: entitytype %s, parentdir %s, entityid %s, altparentdir %s"%
            (cls._entitytype, parent._entitydir, entityid,
                altparent._entitydir if altparent else "(no alt)")
            )
        entity = None
        if util.valid_id(entityid):
            e = cls._child_init(parent, entityid, altparent=altparent, use_altpath=use_altpath)
            v = e._load_values()
            # log.info("entity.load %r"%(v,))
            if v:
                v = e._migrate_values(v)
                e.set_values(v)
                entity = e
        return entity

    @classmethod
    def exists(cls, parent, entityid, altparent=None, use_altpath=False):
        """
        Method tests for existence of identified entity descended from given parent.

        cls         is the class of the entity to be tested
        parent      is the parent from which the entity is descended.
        entityid    is the local identifier (slug) for the entity.
        altparent   is an alternative parent entity to search for the tested entity, 
                    using the alternative path for the entity type.
        use_altpath is set True if this entity is situated at the alternative
                    path relative to its parent.

        Returns True if the entity exists, as determined by existence of the 
        entity description metadata file.
        """
        log.debug("Entity.exists: entitytype %s, parentdir %s, entityid %s"%
            (cls._entitytype, parent._entitydir, entityid)
            )
        e = cls._child_init(parent, entityid, altparent=altparent, use_altpath=use_altpath)
        e._migrate_path()
        return e._exists()

    @classmethod
    def remove(cls, parent, entityid, use_altpath=False):
        """
        Method removes an entity, deleting its details, data and descendents from Annalist storage.

        cls         is the class of the entity to be removed
        parent      is the parent from which the entity is descended.
        entityid    is the local identifier (slug) for the entity.
        use_altpath is set True if this entity is situated at the alternative
                    path relative to its parent.

        Returns None on success, or a status value indicating a reason for value.
        """
        log.debug("Colllection.remove: id %s"%(entityid))
        e = cls.load(parent, entityid, use_altpath=use_altpath)
        if e:
            d = e._entitydir
            # Extra check to guard against accidentally deleting wrong thing
            if cls._entitytype in e['@type'] and d.startswith(parent._entitydir):
                shutil.rmtree(d)
            else:
                log.error("Expected type_id: %s, got %s"%(cls._entitytypeid, e[ANNAL.CURIE.type_id]))
                log.error("Expected dirbase: %s, got %s"%(parent._entitydir, d))
                raise Annalist_Error("Entity %s unexpected type %s or path %s"%(entityid, e[ANNAL.CURIE.type_id], d))
        else:
            return Annalist_Error("Entity %s not found"%(entityid))
        return None

    @classmethod
    def fileobj(cls, parent, entityid, filename, filetypeuri, mimetype, mode, altparent=None, use_altpath=False):
        """
        Method returns a file object value (like `open`) for accessing an imported
        resource associated with an entity (e.g. image, binary blob, etc.)

        cls         is the class of the entity to be tested
        parent      is the parent from which the entity is descended.
        entityid    is the local identifier (slug) for the entity.
        filename    is the local name for the file object to ne created or accessed.
        filetypeuri is a URI or CURIE indicating the type of resource for which a file
                    object is created.  This is used to determine details such as file 
                    extension used when creating a new file.
        mimetype    is a MIME content-type string for the resource representation used.
        mode        indicates how the resource is to be opened, with the same options
                    that are used with the standard `open` method (as far as they are 
                    applicable).  E.g. "wb" to create a new resource, and "r" to read 
                    an existing one.
        altparent   is an alternative parent entity to search for the tested entity, 
                    using the alternative path for the entity type.
        use_altpath is set True if this entity is situated at the alternative
                    path relative to its parent.

        Returns a file object value, or None.
        """
        log.debug("Entity.exists: entitytype %s, parentdir %s, entityid %s"%
            (cls._entitytype, parent._entitydir, entityid)
            )
        e = cls._child_init(parent, entityid, altparent=altparent, use_altpath=use_altpath)
        return e._fileobj(filename, filetypeuri, mimetype, mode)

# End.



#!/usr/bin/python

"""
Context manager for switching the current working directory
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2011-2014, University of Oxford"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import sys
import os

class ChangeCurrentDir:
    """
    Context handler class that swiches the current working directory for some controlled code.
    """
    
    def __init__(self, usecwd):
        self._usecwd = usecwd
        self._oldcwd = None
        return
    
    def __enter__(self):
        self._oldcwd = os.getcwd()
        os.chdir(os.path.join(self._oldcwd, self._usecwd))
        return 

    def __exit__(self, exctype, excval, exctraceback):
        if self._oldcwd:
            os.chdir(self._oldcwd)
        return False

if __name__ == "__main__":
    print os.getcwd()
    oldcwd = os.getcwd()
    with ChangeCurrentDir("test"):
        print os.getcwd()
        assert os.getcwd() == oldcwd+"/test"
    print os.getcwd()
    assert os.getcwd() == oldcwd

# End.



"""
Create Annalist/Django superuser.
"""

from __future__ import print_function

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import os
import sys
import importlib
import re
import logging
# import subprocess

log = logging.getLogger(__name__)

import django

from annalist.identifiers           import ANNAL, RDFS
from annalist.models.annalistuser   import AnnalistUser

from utils.SuppressLoggingContext   import SuppressLogging

import am_errors
from am_settings                    import am_get_settings, am_get_site
from am_getargvalue                 import getarg, getargvalue, getsecret

def create_user_permissions(site, user_id, user_uri, user_name, user_comment, user_permissions):
    user_values = (
        { ANNAL.CURIE.type:             ANNAL.CURIE.User
        , RDFS.CURIE.label:             user_name
        , RDFS.CURIE.comment:           user_comment
        , ANNAL.CURIE.user_uri:         "%s"%(user_uri)
        , ANNAL.CURIE.user_permissions: user_permissions
        })
    user = AnnalistUser.create(site, user_id, user_values, use_altpath=True)
    return user

def delete_user_permissions(site, user_id):
    AnnalistUser.remove(site, user_id, use_altpath=True)
    return

def get_site_settings(annroot, userhome, options):
    """
    Access site settings, set up correspondingh django configuration and return the settings module
    """
    settings = am_get_settings(annroot, userhome, options)
    if not settings:
        print("Settings not found (%s)"%(options.configuration), file=sys.stderr)
        return None
    with SuppressLogging(logging.INFO):
        os.environ['DJANGO_SETTINGS_MODULE'] = settings.modulename
        django.setup()
        site_settings = importlib.import_module(settings.modulename)
    return site_settings

def get_user_name(options, prompt_prefix):
    user_name_regex         = r"^[a-zA-Z0-9@.+_-]+$"
    user_name_prompt        = "%s name:        "%prompt_prefix
    user_name = getargvalue(getarg(options.args, 0), user_name_prompt)
    while not re.match(user_name_regex, user_name):
        print("Invalid username %s - re-enter"%user_name, file=sys.stderr)
        user_name = getargvalue(None, user_name_prompt)
    return user_name

def get_user_details(user_name, options, prompt_prefix):
    """
    Get user details (email, first name, last name)

    Returns a dictionary of user details, including the supplied user_name.
    """
    user_email_regex        = r"^[A-Za-z0-9.+_-]+@([A-Za-z0-9_-]+)(\.[A-Za-z0-9_-]+)*$"
    user_email_prompt       = "%s email:       "%prompt_prefix
    user_first_name_prompt  = "%s first name:  "%prompt_prefix
    user_last_name_prompt   = "%s last name:   "%prompt_prefix
    # Get other values
    user_email = getargvalue(getarg(options.args, 1), user_email_prompt)
    while not re.match(user_email_regex, user_email):
        print("Invalid email address %s - re-enter"%user_email, file=sys.stderr)
        user_email = getargvalue(None, user_email_prompt)
    user_first_name = getargvalue(getarg(options.args, 2), user_first_name_prompt)
    user_last_name  = getargvalue(getarg(options.args, 3), user_last_name_prompt)
    return (
        { 'name':       user_name
        , 'email':      user_email
        , 'uri':        "mailto:%s"%user_email
        , 'first_name': user_first_name
        , 'last_name':  user_last_name
        })

def get_user_permissions(options, pos, prompt_prefix):
    """
    Get user permissions to apply
    """
    user_permissions_regex  = r"^([A-Za-z0-9_-]+(\s+[A-Za-z0-9_-]+)*)?$"
    user_permissions_prompt = "%s permissions: "%prompt_prefix
    user_permissions = getargvalue(getarg(options.args, pos), user_permissions_prompt)
    while not re.match(user_permissions_regex, user_permissions):
        print("Invalid permissions %s - re-enter"%user_permissions, file=sys.stderr)
        user_permissions = getargvalue(None, user_permissions_prompt)
    return user_permissions

def create_django_user(user_type, user_details):
    """
    Create Django user (prompts for password)
    """
    # Get password
    user_password_prompt    = "Password: "
    user_password_c_prompt  = "Re-enter password: "
    user_password   = getsecret(user_password_prompt)
    user_password_c = getsecret(user_password_c_prompt)
    while user_password != user_password_c:
        print("Password values mismatch - try again", file=sys.stderr)
        user_password   = getsecret(user_password_prompt)
        user_password_c = getsecret(user_password_c_prompt)
    # Have all the details - create djano user now
    # Create the user in the Django user database
    # see:
    #   https://docs.djangoproject.com/en/1.7/ref/contrib/auth/#django.contrib.auth.models.User
    #   https://docs.djangoproject.c om/en/1.7/ref/contrib/auth/#manager-methods
    from django.contrib.auth.models import User     # import deferred until after sitesettings import
    user = User.objects.create_user(user_details['name'], user_details['email'], user_password)
    user.first_name   = user_details['first_name']
    user.last_name    = user_details['last_name']
    user.is_active    = True
    user.is_staff     = user_type in ["staff", "superuser"]
    user.is_superuser = user_type in ["superuser"]
    user.save()
    return am_errors.AM_SUCCESS

def create_site_permissions(sitesettings, user_details, permissions):
    site   = am_get_site(sitesettings)
    if not 'label' in user_details:
        user_details['label'] = (
            "%(first_name)s %(last_name)s"%user_details
            )
    if not 'comment' in user_details:
        user_details['comment'] = (
            "User %(name)s: site permissions for %(first_name)s %(last_name)s"%user_details
            )
    user = create_user_permissions(
        site, user_details['name'], user_details['uri'], 
        user_details['label'],
        user_details['comment'], 
        permissions
        )
    return am_errors.AM_SUCCESS

def am_createadminuser(annroot, userhome, options):
    """
    Create Annalist/Django admin/superuser account.  

    Once created, this can be used to create additional users through the 
    site 'admin' link, and can also create collections and

    annroot     is the root directory for theannalist software installation.
    userhome    is the home directory for the host system user issuing the command.
    options     contains options parsed from the command line.

    returns     0 if all is well, or a non-zero status code.
                This value is intended to be used as an exit status code
                for the calling program.
    """
    prompt_prefix = "Admin user"
    if len(options.args) > 4:
        print("Unexpected arguments for %s: (%s)"%(options.command, " ".join(options.args)), file=sys.stderr)
        return am_errors.AM_UNEXPECTEDARGS
    sitesettings = get_site_settings(annroot, userhome, options)
    if not sitesettings:
        return am_errors.AM_NOSETTINGS
    user_name = get_user_name(options, prompt_prefix)
    #
    from django.contrib.auth.models import User     # import deferred until after sitesettings import
    if User.objects.filter(username=user_name):
        print("User %s already exists"%user_name, file=sys.stderr)
        return am_errors.AM_USEREXISTS
    user_details = get_user_details(user_name, options, prompt_prefix)
    status = create_django_user("superuser", user_details)
    if status == am_errors.AM_SUCCESS:
        status = create_site_permissions(
            sitesettings, user_details, 
            ["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
            )
    return status

def am_defaultadminuser(annroot, userhome, options):
    """
    Create default Annalist/Django admin/superuser account.  

    Creates an admin accoubnt with default values:
        username:   admin
        email:      admin@localhost
        firstname:  Admin
        lastname:   User

    Prompts for password as before

    annroot     is the root directory for theannalist software installation.
    userhome    is the home directory for the host system user issuing the command.
    options     contains options parsed from the command line.

    returns     0 if all is well, or a non-zero status code.
                This value is intended to be used as an exit status code
                for the calling program.
    """
    if len(options.args) > 0:
        print("Unexpected arguments for %s: (%s)"%(options.command, " ".join(options.args)), file=sys.stderr)
        return am_errors.AM_UNEXPECTEDARGS
    sitesettings = get_site_settings(annroot, userhome, options)
    if not sitesettings:
        return am_errors.AM_NOSETTINGS
    default_admin = (
        { 'name':       "admin"
        , 'email':      "admin@localhost"
        , 'uri':        "mailto:admin@localhost"
        , 'first_name': "Admin"
        , 'last_name':  "User"
        })
    print("Creating user %(name)s"%default_admin, file=sys.stderr)
    status = create_django_user("superuser", default_admin)
    if status == am_errors.AM_SUCCESS:
        status = create_site_permissions(
            sitesettings, default_admin, 
            ["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
            )
    return status

def am_updateadminuser(annroot, userhome, options):
    """
    Update existing Django user to admin status

    annroot     is the root directory for the Annalist software installation.
    userhome    is the home directory for the host system user issuing the command.
    options     contains options parsed from the command line.

    returns     0 if all is well, or a non-zero status code.
                This value is intended to be used as an exit status code
                for the calling program.
    """
    # see:
    #   https://docs.djangoproject.com/en/1.7/ref/contrib/auth/#django.contrib.auth.models.User
    prompt_prefix = "Update user"
    if len(options.args) > 1:
        print("Unexpected arguments for %s: (%s)"%(options.command, " ".join(options.args)), file=sys.stderr)
        return am_errors.AM_UNEXPECTEDARGS
    sitesettings = get_site_settings(annroot, userhome, options)
    if not sitesettings:
        return am_errors.AM_NOSETTINGS
    user_name = get_user_name(options, prompt_prefix)
    #
    from django.contrib.auth.models import User     # import deferred until after sitesettings import
    userqueryset = User.objects.filter(username=user_name)
    if not userqueryset:
        print("User %s does not exist"%user_name, file=sys.stderr)
        return am_errors.AM_USERNOTEXISTS
    # Have all the details - now update the user in the Django user database
    # see:
    #   https://docs.djangoproject.com/en/1.7/ref/contrib/auth/#django.contrib.auth.models.User
    #   https://docs.djangoproject.c om/en/1.7/ref/contrib/auth/#manager-methods
    user = userqueryset[0]
    user.is_staff     = True
    user.is_superuser = True
    user.save()
    # Create site permissions record for admin user
    user_details = (
        { 'name':       user_name
        , 'email':      user.email
        , 'uri':        "mailto:%s"%user.email
        , 'first_name': user.first_name
        , 'last_name':  user.last_name
        })
    status = create_site_permissions(
        sitesettings, user_details, 
        ["VIEW", "CREATE", "UPDATE", "DELETE", "CONFIG", "ADMIN"]
        )
    return status

def am_setdefaultpermissions(annroot, userhome, options):
    """
    Set site-wide default permissions for logged in users

    annroot     is the root directory for theannalist software installation.
    userhome    is the home directory for the host system user issuing the command.
    options     contains options parsed from the command line.

    returns     0 if all is well, or a non-zero status code.
                This value is intended to be used as an exit status code
                for the calling program.
    """
    prompt_prefix = "Default "
    if len(options.args) > 1:
        print("Unexpected arguments for %s: (%s)"%(options.command, " ".join(options.args)), file=sys.stderr)
        return am_errors.AM_UNEXPECTEDARGS
    sitesettings = get_site_settings(annroot, userhome, options)
    if not sitesettings:
        return am_errors.AM_NOSETTINGS
    user_permissions = get_user_permissions(options, 0, prompt_prefix)
    user_details = (
        { 'name':       "_default_user_perms"
        , 'uri':        "annal:User/_default_user_perms"
        , 'label':      "Default permissions"
        , 'comment':    "Default permissions for authenticated user."
        })
    status = create_site_permissions(sitesettings, user_details, user_permissions)
    return status

def am_setpublicpermissions(annroot, userhome, options):
    """
    Set site-wide default permissions for unauthenticated public access

    annroot     is the root directory for the Annalist software installation.
    userhome    is the home directory for the host system user issuing the command.
    options     contains options parsed from the command line.

    returns     0 if all is well, or a non-zero status code.
                This value is intended to be used as an exit status code
                for the calling program.
    """
    prompt_prefix = "Public access "
    if len(options.args) > 1:
        print("Unexpected arguments for %s: (%s)"%(options.command, " ".join(options.args)), file=sys.stderr)
        return am_errors.AM_UNEXPECTEDARGS
    sitesettings = get_site_settings(annroot, userhome, options)
    if not sitesettings:
        return am_errors.AM_NOSETTINGS
    user_permissions = get_user_permissions(options, 0, prompt_prefix)
    user_details = (
        { 'name':       "_unknown_user_perms"
        , 'uri':        "annal:User/_unknown_user_perms"
        , 'label':      "Unknown user"
        , 'comment':    "Permissions for unauthenticated user."
        })
    status = create_site_permissions(sitesettings, user_details, user_permissions)
    return status

def am_deleteuser(annroot, userhome, options):
    """
    Delete Annalist/Django user account.  

    annroot     is the root directory for theannalist software installation.
    userhome    is the home directory for the host system user issuing the command.
    options     contains options parsed from the command line.

    returns     0 if all is well, or a non-zero status code.
                This value is intended to be used as an exit status code
                for the calling program.
    """
    prompt_prefix = "Delete user"
    if len(options.args) > 1:
        print("Unexpected arguments for %s: (%s)"%(options.command, " ".join(options.args)), file=sys.stderr)
        return am_errors.AM_UNEXPECTEDARGS
    sitesettings = get_site_settings(annroot, userhome, options)
    if not sitesettings:
        return am_errors.AM_NOSETTINGS
    user_name = get_user_name(options, prompt_prefix)
    #
    from django.contrib.auth.models import User     # import deferred until after sitesettings import
    userqueryset = User.objects.filter(username=user_name)
    if not userqueryset:
        print("User %s does not exist"%user_name, file=sys.stderr)
        return am_errors.AM_USERNOTEXISTS
    userqueryset.delete()
    site = am_get_site(sitesettings)
    delete_user_permissions(site, user_name)
    return am_errors.AM_SUCCESS

# End.



"""
Annalist action confirmation view definition
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import os.path
import json
import random
import logging
import uuid
import copy

import logging
log = logging.getLogger(__name__)

# import rdflib
# import httplib2

from django.http                    import HttpResponse
from django.http                    import HttpResponseRedirect
from django.http                    import QueryDict
from django.template                import RequestContext, loader
from django.core.urlresolvers       import resolve, reverse

from django.conf                    import settings

from annalist.views.generic         import AnnalistGenericView

def querydict_dict(querydict):
    """
    Converts a Django QueryDict value to a regular dictionary, preserving multiple items.
    """
    return dict(querydict.iterlists())

def dict_querydict(dict_):
    """
    Converts a value created by querydict_dict back into a Django QueryDict value.
    """
    q = QueryDict("", mutable=True)
    for k, v in dict_.iteritems():
        q.setlist(k, v)
    q._mutable = False
    return q

def querydict_dumps(querydict):
    """
    Converts a Django QueryDict value to a serialized form, preserving multiple items.

    Serializes as JSON where each key has a list value.
    """
    return json.dumps(querydict_dict(querydict))

def querydict_loads(querydict_s):
    """
    Converts a serialized Django QueryDict back to its internal value.
    """
    return dict_querydict(json.loads(querydict_s))

class ConfirmView(AnnalistGenericView):
    """
    View class to handle response to request to confirm action
    """
    def __init__(self):
        super(ConfirmView, self).__init__()
        return

    @staticmethod
    def render_form(request,
            action_description="Are you sure you want to do that?", # message describing requested action
            action_params={},
            confirmed_action_uri="/",    # URI to to POST to complete action
            cancel_action_uri="/",      # URI to dispatch to cancel action
            title=None):
        """
        Render form that requests a user to confirm an action to be performed and,
        depending onthe user's response, redirects to 'confirmed_action_uri' or 
        'cancel_action_uri'
        """
        form_data = (
            { "action_description":     action_description
            , "action_params":          querydict_dumps(action_params)
            , "confirmed_action":       confirmed_action_uri
            , "cancel_action":          cancel_action_uri
            , "title":                  title
            , "suppress_user":          True
            })
        template = loader.get_template('annalist_confirm.html')
        context  = RequestContext(request, form_data)
        log.debug("confirmview form data: %r"%(form_data))
        return HttpResponse(template.render(context))

    # POST

    def post(self, request):
        """
        Handle POST of form data soliciting user confirmation of action.

        Creates a new request object with the original action POST data, and dispatches
        to the appropriate completion view function.  This function should, in turn,
        return a redirect to an appripriate continuation display.

        If the operation is canceled, then this function returns an HTTP redirect to 
        the "cancel_action" continuation URI.
        """
        log.debug("confirmview.post: %r"%(request.POST))
        if "confirm" in request.POST:
            action_request      = copy.copy(request)
            action_request.POST = querydict_loads(request.POST["action_params"])
            view, args, kwargs  = resolve(request.POST["confirmed_action"])
            return view(action_request, *args, **kwargs)
        return HttpResponseRedirect(request.POST["cancel_action"])

# End.



"""
Analist site-related facilities
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import os
import os.path
import collections
import urlparse
import json
from collections    import OrderedDict

import traceback
import logging
log = logging.getLogger(__name__)

from django.http                    import HttpResponse
from django.http                    import HttpResponseRedirect
from django.conf                    import settings
from django.core.urlresolvers       import resolve, reverse

import annalist
from annalist.identifiers           import RDF, RDFS, ANNAL
from annalist.exceptions            import Annalist_Error, EntityNotFound_Error
from annalist.util                  import valid_id, extract_entity_id
from annalist                       import layout
from annalist                       import message

from annalist.models.annalistuser   import AnnalistUser
from annalist.models.entityroot     import EntityRoot
from annalist.models.sitedata       import SiteData
from annalist.models.collection     import Collection
from annalist.models.recordvocab    import RecordVocab
from annalist.models.recordview     import RecordView
from annalist.models.recordgroup    import RecordGroup
from annalist.models.recordfield    import RecordField

# @@TODO: look for way to remove this view dependency from model code.  Move to RecordField 
#   module?  Hmmm... not so easy: information about render types and methods is concentrated
#   in render_utils module.  Maybe need to split render_utils functionality between
#   model and view modules?
#
from annalist.views.fields.render_utils import (
    is_render_type_literal,
    is_render_type_id,
    is_render_type_set,
    is_render_type_list,
    is_render_type_object,
    )

class Site(EntityRoot):

    _entitytype     = ANNAL.CURIE.Site
    _entitytypeid   = "_site"
    _entityfile     = layout.SITE_META_FILE
    _entityref      = layout.META_SITE_REF

    def __init__(self, sitebaseuri, sitebasedir, host=""):
        """
        Initialize a Site object

        sitebaseuri     the base URI of the site
        sitebasedir     the base directory for site information
        """
        log.debug("Site init: %s"%(sitebasedir))
        super(Site, self).__init__(host+sitebaseuri, sitebasedir)
        self._sitedata = SiteData(self)
        return

    def get_user_permissions(self, user_id, user_uri):
        """
        Get a site-wide user permissions record (AnnalistUser).

        To return a value, both the user_id and the user_uri (typically a mailto: URI, but
        may be any *authenticated* identifier) must match.  This is to prevent access to 
        records of a deleted account being granted to a new account created with the 
        same user_id (username).

        user_id         local identifier for the type to retrieve.
        user_uri        authenticated identifier associated with the user_id.  That is,
                        the authentication service used is presumed to confirm that
                        the identifier belongs to the user currently logged in with
                        the supplied username.

        returns an AnnalistUser object for the identified user, or None.  This object contains
                information about permissions granted to the user in the current collection.
        """
        user = AnnalistUser.load(self, user_id, use_altpath=True)
        log.debug(
            "Site.get_user_permissions: user_id %s, user_uri %s, user %r"%
            (user_id, user_uri, user)
            )
        if user:
            for f in [RDFS.CURIE.label, RDFS.CURIE.comment, ANNAL.CURIE.user_uri, ANNAL.CURIE.user_permissions]:
                if f not in user:
                    user = None
                    break
        if user and user[ANNAL.CURIE.user_uri] != user_uri:
            user = None         # URI mismatch: return None.
        return user

    def collections(self):
        """
        Generator enumerates and returns collection descriptions that are part of a site.

        Yielded values are collection objects.
        """
        log.debug("site.collections: basedir: %s"%(self._entitydir))
        for f in self._children(Collection):
            c = Collection.load(self, f)
            # log.info("Site.colections: Collection.load %s %r"%(f, c.get_values()))
            if c:
                yield c
        return

    def collections_dict(self):
        """
        Return an ordered dictionary of collection URIs indexed by collection id
        """
        coll = [ (c.get_id(), c) for c in self.collections() ]
        return collections.OrderedDict(sorted(coll))

    def site_data(self):
        """
        Return dictionary of site data
        """
        # @@TODO: consider using generic view logic for this mapping (and elsewhere?)
        #         This is currently a bit of a kludge, designed to match the site
        #         view template.  In due course, it may be reviewed and implemented
        #         using the generic Annalist form generating framework
        site_data = self._load_values()
        if not site_data:
            return None
        site_data["title"] = site_data.get(RDFS.CURIE.label, message.SITE_NAME_DEFAULT)
        # log.info("site.site_data: site_data %r"%(site_data))
        colls = collections.OrderedDict()
        for k, v in self.collections_dict().items():
            # log.info("site.site_data: colls[%s] %r"%(k, v))
            colls[k] = dict(v.items(), id=k, url=v[ANNAL.CURIE.url], title=v[RDFS.CURIE.label])
        site_data["collections"] = colls
        return site_data

    def add_collection(self, coll_id, coll_meta, annal_ver=annalist.__version_data__):
        """
        Add a new collection to the current site

        coll_id     identifier for the new collection, as a string
                    with a form that is valid as URI path segment.
        coll_meta   a dictionary providing additional information about
                    the collection to be created.
        annal_ver   Override annalist version stored in collection metadata
                    (parameter provided for testing)

        returns a Collection object for the newly created collection.
        """
        d = dict(coll_meta)
        d[ANNAL.CURIE.software_version] = annal_ver
        c = Collection.create(self, coll_id, d)
        return c

    def remove_collection(self, coll_id):
        """
        Remove a collection from the site data.

        coll_id     identifier for the collection to remove.

        Returns a non-False status code if the collection is not removed.
        """
        log.debug("remove_collection: %s"%(coll_id))
        return Collection.remove(self, coll_id)

    # JSON-LD context data

    def generate_site_jsonld_context(self):
        """
        (Re)generate JSON-LD context description for the current collection.

        get_field_uri_context
                is a supplied function that accepts a RecordField object abnd
                returns a context dictionary for the field thus described.
        """
        # Build context data
        context = self.get_site_jsonld_context()
        # Assemble and write out context description
        with self._metaobj("", layout.SITEDATA_CONTEXT_FILE, "wt") as context_io:
            json.dump(
                { "@context": context }, 
                context_io, indent=2, separators=(',', ': ')
                )
        return

    def get_site_jsonld_context(self):
        """
        Return dictionary containing context structure for collection.
        """
        # @@REVIEW: as a workaround for a problem with @base handling in rdflib-jsonld, don't
        #           include @base in context.
        #
        # context           = OrderedDict(
        #     { "@base": self.get_url() + layout.SITEDATA_DIR + "/"
        #     })
        # context           = OrderedDict(
        #     { "@base": "./"
        #     })
        #
        # Use OrderedDict to allow some control over ordering of context file contents:
        # this is for humane purposes only, and is not technically important.
        context           = OrderedDict()
        # Scan vocabs, generate prefix data
        for v in self._site_children(RecordVocab):
            vid = v.get_id()
            if vid != "_initial_values":
                context[v.get_id()] = v[ANNAL.CURIE.uri]
        # Set type info for predefined URI fields
        # for f in [ANNAL.CURIE.user_uri]:
        #     self.set_field_uri_jsonld_context(f, { "@type": "@id" }, context)
        # Scan view fields and generate context data for property URIs used
        for v in self._site_children(RecordView):
            for fref in v[ANNAL.CURIE.view_fields]:
                fid  = extract_entity_id(fref[ANNAL.CURIE.field_id])
                vuri = fref.get(ANNAL.CURIE.property_uri, None)
                furi, fcontext = self.get_field_uri_jsonld_context(fid, self.get_field_jsonld_context)
                # fcontext['vid'] = v.get_id()
                # fcontext['fid'] = fid
                self.set_field_uri_jsonld_context(vuri or furi, fcontext, context)
        # Scan group fields and generate context data for property URIs used
        for g in self._site_children(RecordGroup):
            for gref in g[ANNAL.CURIE.group_fields]:
                fid  = extract_entity_id(gref[ANNAL.CURIE.field_id])
                guri = gref.get(ANNAL.CURIE.property_uri, None)
                furi, fcontext = self.get_field_uri_jsonld_context(fid, self.get_field_jsonld_context)
                # fcontext['gid'] = g.get_id()
                # fcontext['fid'] = fid
                self.set_field_uri_jsonld_context(guri or furi, fcontext, context)
        return context

    def get_field_uri_jsonld_context(self, fid, get_field_context):
        """
        Access field description, and return field property URI and appropriate 
        property description for JSON-LD context.

        If there is no corresponding field description, returns (None, None)

        If no context should be generated for the field URI, returns (uri, None)
        """
        f = RecordField.load(self, fid, use_altpath=True)
        if f is None:
            return (None, None)
        return (f[ANNAL.CURIE.property_uri], get_field_context(f))

    # @@TODO: Make static and use common copy with `collection`
    def set_field_uri_jsonld_context(self, puri, fcontext, property_contexts):
        """
        Save property context description into supplied property_contexts dictionary.  
        If the context is already defined, generate warning if there is a compatibility 
        problem.
        """
        if puri:
            uri_parts = puri.split(":")
            if len(uri_parts) > 1:
                # Ignore URIs without ':'
                if puri in property_contexts:
                    pcontext = property_contexts[puri]
                    if ( ( not fcontext ) or
                         ( pcontext.get("@type", None)      != fcontext.get("@type", None) ) or
                         ( pcontext.get("@container", None) != fcontext.get("@container", None) ) ):
                        log.warning(
                            "Incompatible use of property %s (%r, %r)"% (puri, fcontext, pcontext)
                            )
                elif ( fcontext and
                       ( uri_parts[0] in property_contexts ) or         # Prefix defined vocab?
                       ( uri_parts[0] in ["http", "https", "file"] ) ): # Full URI?
                    property_contexts[puri] = fcontext
        return

    # @@TODO: move this away from model logic, as it represents a dependency on view logic?
    @staticmethod
    def get_field_jsonld_context(fdesc):
        """
        Returns a context description for the supplied field description.

        Returns None if no property context information is needed for the 
        supplied field.
        """
        rtype = extract_entity_id(fdesc[ANNAL.CURIE.field_render_type])
        vmode = extract_entity_id(fdesc[ANNAL.CURIE.field_value_mode])
        if vmode in ["Value_entity", "Value_field"]:
            rtype = "Enum"
        elif vmode == "Value_import":
            rtype = "URIImport"
        elif vmode == "Value_upload":
            rtype = "FileUpload"
        if is_render_type_literal(rtype):
            fcontext = None # { "@type": "xsd:string" }
        elif is_render_type_id(rtype):
            fcontext = { "@type": "@id" }   # Add type from field descr?
        elif is_render_type_set(rtype):
            fcontext = { "@container": "@set"}
        elif is_render_type_list(rtype):
            fcontext = { "@container": "@list"}
        elif is_render_type_object(rtype):
            fcontext = None
        else:
            raise ValueError("Unexpected value mode or render type (%s, %s)"%(vmode, rtype))
        return fcontext

    # Temporary helper functions
    #
    # @@TODO - temporary fix
    #
    # These helpers return children of the site object.  
    #
    # It works like the EntityRoot._children method except that it uses the 
    # alternative parent path for the supplied class.  This alternative path
    # logic should be eliminated when the site data is moved to a special
    # site data collection: then the access for all child data should be
    # unified.

    def _site_children_ids(self, cls):
        """
        Iterates over candidate child entyity ids that are possible instances 
        of an indicated class.  The supplied class is used to determine a 
        subdirectory to be scanned.

        cls         is a subclass of Entity indicating the type of children to
                    iterate over.
        """
        site_dir = os.path.dirname(os.path.join(self._entitydir, cls._entityaltpath))
        assert "%" not in site_dir, "_entitypath/_entityaltpath template variable interpolation may be in filename part only"
        site_files = []
        if site_dir and os.path.isdir(site_dir):
            site_files = os.listdir(site_dir)
        for fil in site_files:
            if valid_id(fil):
                yield fil
        return

    def _site_children(self, cls):
        """
        Iterates over candidate children of the current site object that are instances 
        of an indicated class.

        cls         is a subclass of Entity indicating the type of children to
                    iterate over.
        """
        for i in self._site_children_ids(cls):
            e = cls.load(self, i, use_altpath=True)
            if e:
                yield e
        return

# End.



"""
Renderer and value mapper for field consisting of a list of simple token values.

Simple tokens may not contain whitespace.
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import logging
log = logging.getLogger(__name__)

from annalist.views.fields.render_base          import RenderBase
from annalist.views.fields.render_fieldvalue    import (
    RenderFieldValue,
    get_context_value, get_context_field_value, 
    get_field_edit_value,
    get_field_view_value
    )

from django.template    import Template, Context

#   ----------------------------------------------------------------------------
#
#   Token set render support functions
#
#   ----------------------------------------------------------------------------


class TokenSetValueMapper(RenderBase):
    """
    Value mapper class for token list
    """

    @classmethod
    def encode(cls, field_value):
        """
        Encodes a token list as a string of space-separated values
        """
        if isinstance(field_value, (list,tuple)):
            return " ".join(field_value)
        log.warning("TokenSetValueMapper.encode tokenset: supplied value is not a list or tuple")
        log.warning("TokenSetValueMapper.encode value: %r"%(field_value,))
        return str(field_value)

    @classmethod
    def decode(cls, field_value):
        """
        Decodes a string of space-separated tokens as a list of tokens
        """
        if isinstance(field_value,(str,unicode)):
            return field_value.split()
        log.warning("TokenSetValueMapper.decode: %r"%(field_value,))
        return [field_value]

#   ----------------------------------------------------------------------------
#
#   Token set field renderers
#
#   ----------------------------------------------------------------------------

class tokenset_view_renderer(object):
    def render(self, context):
        """
        Render token list for viewing.
        """
        tokenset = get_field_view_value(context, None)
        return TokenSetValueMapper.encode(tokenset) if tokenset else "&nbsp;"

class tokenset_edit_renderer(object):

    def __init__(self):
        self._template = Template(
            '''<input type="text" size="64" name="{{repeat_prefix}}{{field.field_name}}" '''+
                '''placeholder="{{field.field_placeholder}}" '''+
                '''value="{{encoded_field_value}}"/>'''
            )

    def render(self, context):
        """
        Render token list for editing
        """
        tokenset = get_field_edit_value(context, None)
        with context.push(encoded_field_value=TokenSetValueMapper.encode(tokenset)):
            return self._template.render(context)

def get_field_tokenset_renderer():
    """
    Return field renderer object for token list values
    """
    return RenderFieldValue(
        view_renderer=tokenset_view_renderer(), 
        edit_renderer=tokenset_edit_renderer(),
        )

# End.



#!/usr/bin/env python
#
# NOTE: when testing, use "pip install ... --upgrade"

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2011-2013, University of Oxford"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

# Setup.py based on https://github.com/paltman/python-setup-template/blob/master/setup.py,
# following http://www.ibm.com/developerworks/opensource/library/os-pythonpackaging/index.html
#
# These could be useful:
#   https://wiki.python.org/moin/Distutils/Tutorial
#   https://pypi.python.org/pypi/check-manifest


import codecs
import os
import sys

from distutils.util import convert_path
from fnmatch import fnmatchcase
from setuptools import setup, find_packages
from pip.req import parse_requirements      # See: http://stackoverflow.com/questions/14399534/

if sys.version_info[:2] != (2,7):
    raise AssertionError("Annalist requires Python 2.7 (found Python %s.%s)"%sys.version_info[:2])

dir_here = os.path.dirname(__file__)
sys.path.insert(0, os.path.join(dir_here, "annalist_root"))

# Helper to load README.rst, etc.
def read(fname):
    return codecs.open(os.path.join(dir_here, fname)).read()

PACKAGE         = "annalist"
PACKAGE_MODULE  = __import__(PACKAGE, globals(), locals(), ['__version__', '__author__'])
VERSION         = PACKAGE_MODULE.__version__
AUTHOR          = PACKAGE_MODULE.__author__
AUTHOR_EMAIL    = "gk-pypi@ninebynine.org"
NAME            = "Annalist"
DESCRIPTION     = "Annalist linked data notebook"
URL             = "https://github.com/gklyne/annalist"

setup(
    name = NAME,
    version = VERSION,
    description = DESCRIPTION,
    long_description = read("README.rst"),
    author = AUTHOR,
    author_email = AUTHOR_EMAIL,
    license = "MIT",
    url = URL,
    packages = 
        [ 'annalist_root'
          , 'annalist_root.annalist'
            , 'annalist_root.annalist.models'
            , 'annalist_root.annalist.views'
            , 'annalist_root.annalist.views.fields'
            , 'annalist_root.annalist.views.form_utils'
            , 'annalist_root.annalist.tests'
          , 'annalist_root.annalist_site'
            , 'annalist_root.annalist_site.settings'
          , 'annalist_root.annalist_manager'
          , 'annalist_root.utils'
          , 'annalist_root.oauth2'
          , 'annalist_root.miscutils'
        ],
    package_dir = 
        { 'annalist_root':  'annalist_root'
        # , 'annalist':       'annalist_root/annalist'
        # , 'annalist_site':  'annalist_root/annalist_site'
        # , 'utils':          'annalist_root/annalist'
        # , 'oauth2':         'annalist_root/utils'
        # , 'miscutils':      'annalist_root/miscutils'
        },
    # >>>> REMEMBER to also update MANIFEST.in ... <<<<
    package_data = 
        { 'annalist_root':
            [ '*.sh', '*.txt'
            , 'sampledata/README.md'
            , 'sampledata/init/annalist_site/README.md'
            , 'sampledata/init/annalist_site/test-image.jpg'
            , 'sampledata/init/annalist_site/_annalist_site/*.jsonld'
            , 'sampledata/init/annalist_site/c/*/_annalist_collection/*.jsonld'
            , 'sampledata/init/annalist_site/c/*/_annalist_collection/lists/*/*.jsonld'
            , 'sampledata/init/annalist_site/c/*/_annalist_collection/types/*/*.jsonld'
            , 'sampledata/init/annalist_site/c/*/_annalist_collection/views/*/*.jsonld'
            , 'sampledata/init/annalist_site/c/*/d/*/*/*.jsonld'
            , 'sampledata/empty/annalist_site/README.md'
            , 'sampledata/empty/annalist_site/_annalist_site/*.jsonld'
            , 'sampledata/empty/annalist_site/c/README.md'
            ]
        , 'annalist_root.annalist':
            [ 'templates/*.html'
            , 'templates/field/*.html'
            , 'static/css/*.css'
            , 'static/js/*.js'
            , 'static/images/*.png'
            , 'static/images/icons/warning_32.png'
            , 'static/images/icons/search_32.png'
            , 'static/foundation/css/*.css'
            # , 'static/foundation/img/*.png'
            , 'static/foundation/js/foundation/*.js'
            , 'static/foundation/js/vendor/*.js'
            , 'sitedata/enums/*/*/*.jsonld'
            , 'sitedata/fields/*/*.jsonld'
            , 'sitedata/groups/*/*.jsonld'
            , 'sitedata/lists/*/*.jsonld'
            , 'sitedata/types/*/*.jsonld'
            , 'sitedata/views/*/*.jsonld'
            , 'sitedata/users/*/*.jsonld'
            , 'sitedata/vocabs/*/*.jsonld'
            ]
        , 'annalist_root.annalist.views':
            [ 'help/*.md'
            , 'help/*.html'
            ]
        , 'annalist_root.oauth2':
            [ 'templates/*.html'
            ]
        },
    exclude_package_data = {
        '': ['spike/*'] 
        },
    data_files = 
        [
        ],
    classifiers=
        [
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        ],
    zip_safe = False,
    install_requires =
        [ 'Django==1.7'
          , 'wsgiref==0.1.2'
        , 'oauth2client==1.2'
          , 'httplib2==0.9'
        , 'pyparsing==2.0.2'
        , 'Markdown==2.5.2'
        # For testing:
        , 'httpretty==0.8.10'
        , 'beautifulsoup4==4.4.0'
          , 'html5lib==1.0b8'
        , 'rdflib==4.2.1'
        , 'rdflib-jsonld==0.3'
          , 'SPARQLWrapper==1.6.4'
          , 'isodate==0.5.1'
          , 'wsgiref==0.1.2'
          , 'six==1.10.0'
        ],
    entry_points =
        {
        'console_scripts':
            [ 'annalist-manager = annalist_root.annalist_manager.am_main:runMain',
            ]
        }
    )

# End.



"""
Collection of Annalist data records for a specified record type
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import os
import os.path
import urlparse
import shutil

import logging
log = logging.getLogger(__name__)

from django.conf import settings

from annalist                   import layout
from annalist.exceptions        import Annalist_Error
from annalist.identifiers       import ANNAL
from annalist                   import util
from annalist.models.entity     import Entity
from annalist.models.entitydata import EntityData

class RecordTypeData(Entity):

    _entitytype     = ANNAL.CURIE.Type_Data
    _entitytypeid   = "_entitytypedata"
    _entityview     = layout.COLL_TYPEDATA_VIEW
    _entitypath     = layout.COLL_TYPEDATA_PATH
    _entityfile     = layout.TYPEDATA_META_FILE
    _entityref      = layout.META_TYPEDATA_REF
    _contextref     = layout.COLL_CONTEXT_FILE

    def __init__(self, parent, type_id):
        """
        Initialize a new RecordTypeData object, without metadata.

        parent      is the parent collection from which the type data is descended.
        type_id     the local identifier (slug) for the record type
        """
        super(RecordTypeData, self).__init__(parent, type_id)
        return

    # @@TODO remove this method and re-test (now redundant, handled through Entity class)
    def entities(self):
        """
        Generator enumerates and returns records of given type
        """
        for f in self._children(EntityData):
            log.debug("RecordTypeData.entities: f %s"%f)            
            e = EntityData.load(self, f)
            if e:
                yield e
        return

    def remove_entity(self, entity_id):
        t = EntityData.remove(self, entity_id)
        return t

# End.



"""
Annalist site data

Site data is an alternative location for generic Annalist metatadata 
(e.g. type and view definitions, etc.) that are common across all 
collections (and even installations).
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import os
import os.path
import urlparse
import shutil

import logging
log = logging.getLogger(__name__)

from django.conf import settings

from annalist                   import layout
from annalist.exceptions        import Annalist_Error
from annalist.identifiers       import ANNAL
from annalist                   import util

from annalist.models.entity     import Entity
from annalist.models.recordtype import RecordType
from annalist.models.recordview import RecordView
from annalist.models.recordlist import RecordList

class SiteData(Entity):

    _entitytype     = ANNAL.CURIE.SiteData
    _entitytypeid   = "_sitedata"
    _entityview     = layout.SITEDATA_VIEW
    _entitypath     = layout.SITEDATA_PATH
    _entityfile     = layout.SITEDATA_META_FILE
    _entityref      = layout.META_SITEDATA_REF

    def __init__(self, parent):
        """
        Initialize a new SiteData object, without metadta (yet).

        parent      is the parent site from which the new collection is descended.
        """
        super(SiteData, self).__init__(parent, layout.SITEDATA_DIR, idcheck=False)
        return

    # Record types

    def types(self):
        """
        Generator enumerates and returns record types that may be stored
        """
        for f in self._children(RecordType):
            t = RecordType.load(self, f)
            if t:
                yield t
        return

    # Record views

    def views(self):
        """
        Generator enumerates and returns record views that may be stored
        """
        for f in self._children(RecordView):
            t = RecordView.load(self, f)
            if t:
                yield t
        return

    # Record lists

    def lists(self):
        """
        Generator enumerates and returns record lists that may be stored
        """
        for f in self._children(RecordList):
            t = RecordList.load(self, f)
            if t:
                yield t
        return

# End.



"""
Test module used to create some initial site data for experimentation and manual testing
"""

__author__      = "Graham Klyne (GK@ACM.ORG)"
__copyright__   = "Copyright 2014, G. Klyne"
__license__     = "MIT (http://opensource.org/licenses/MIT)"

import os
import unittest

import logging
log = logging.getLogger(__name__)

from django.conf                    import settings
from django.test                    import TestCase # cf. https://docs.djangoproject.com/en/dev/topics/testing/tools/#assertions

from annalist.identifiers           import RDF, RDFS, ANNAL
from annalist                       import layout
from annalist.models.site           import Site
from annalist.models.collection     import Collection
from annalist.models.recordtype     import RecordType
from annalist.models.recordview     import RecordView
from annalist.models.recordlist     import RecordList
from annalist.models.recordtypedata import RecordTypeData
from annalist.models.entitydata     import EntityData

from annalist.views.defaultedit     import EntityDefaultEditView

from AnnalistTestCase       import AnnalistTestCase
from tests                  import TestHost, TestHostUri, TestBasePath, TestBaseUri, TestBaseDir
from tests                  import test_layout
from init_tests             import init_annalist_test_site, init_annalist_test_coll, resetSitedata
from init_tests             import copySitedata

from AnnalistTestCase               import AnnalistTestCase
from entity_testutils               import collection_create_values
from entity_testtypedata            import recordtype_create_values
from entity_testviewdata            import recordview_create_values
from entity_testlistdata            import recordlist_create_values

#   -----------------------------------------------------------------------------
#
#   Helper functions
#
#   -----------------------------------------------------------------------------

def entitydata_create_values(coll, etype, entity_id, update="Entity"):
    """
    Data used when creating entity test data
    """
    return (
        { 'rdfs:label': '%s %s/%s/%s'%(update, coll._entityid, etype._entityid, entity_id)
        , 'rdfs:comment': '%s coll %s, type %s, entity %s'%(update, coll._entityid, etype._entityid, entity_id)
        })

def coll123_create_data(site):
    coll1 = Collection.create(site, "coll1", collection_create_values("coll1"))
    coll2 = Collection.create(site, "coll2", collection_create_values("coll2"))
    coll3 = Collection.create(site, "coll3", collection_create_values("coll3"))
    #
    for coll in [coll1, coll2, coll3]:
        type1 = RecordType.create(coll, "type1", recordtype_create_values(coll._entityid, "type1"))
        view1 = RecordView.create(coll, "view1", recordview_create_values(coll._entityid, "view1"))
        list1 = RecordList.create(coll, "list1", recordlist_create_values(coll._entityid, "list1"))
        data1 = RecordTypeData.create(coll, "type1", {})
        type2 = RecordType.create(coll, "type2", recordtype_create_values(coll._entityid, "type2"))
        view2 = RecordView.create(coll, "view2", recordview_create_values(coll._entityid, "view2"))
        list2 = RecordList.create(coll, "list2", recordlist_create_values(coll._entityid, "list2"))
        data2 = RecordTypeData.create(coll, "type2", {})
        #
        for t,d in [(type1,data1),(type2,data2)]:
            for eid in ["entity1", "entity2", "entity3"]:
                e = EntityData.create(d, eid, entitydata_create_values(coll,t,eid))
    return

#   -----------------------------------------------------------------------------
#
#   CreateSiteData
#
#   -----------------------------------------------------------------------------

class CreateSiteData(AnnalistTestCase):
    """
    Tests for Site object interface
    """

    def setUp(self):
        return

    def tearDown(self):
        return

    #   -----------------------------------------------------------------------------
    #   Create site data
    #   -----------------------------------------------------------------------------

    def test_CreateDevelSiteData(self):
        copySitedata(
            settings.SITE_SRC_ROOT+"/sampledata/init/"+test_layout.SITE_DIR, 
            settings.SITE_SRC_ROOT+"/annalist/sitedata",
            TestBaseDir)
        develsite = Site("http://localhost:8000/annalist/", TestBaseDir)
        coll123_create_data(develsite)
        return

    def test_CreateTestSiteData(self):
        copySitedata(
            settings.SITE_SRC_ROOT+"/sampledata/init/"+test_layout.SITE_DIR, 
            settings.SITE_SRC_ROOT+"/annalist/sitedata",
            TestBaseDir)
        testsite = Site(TestBaseUri, TestBaseDir)
        coll123_create_data(testsite)
        #
        # testcoll = Collection.create(testsite, "testcoll", collection_create_values("testcoll"))
        # testtype = RecordType.create(testcoll, "testtype", recordtype_create_values("testcoll", "testtype"))
        # # testview = RecordView.create(testcoll, "testview", recordview_create_values("testview"))
        # # testlist = RecordList.create(testcoll, "testlist", recordlist_create_values("testlist"))
        # testdata = RecordTypeData.create(testcoll, "testtype", {})
        # teste    = EntityData.create(
        #     testdata, "entity1", 
        #     entitydata_create_values(testcoll,testtype,"entity1")
        #     )
        return

    def test_CreateEmptySiteData(self):
        copySitedata(
            settings.SITE_SRC_ROOT+"/sampledata/empty/"+test_layout.SITE_DIR, 
            settings.SITE_SRC_ROOT+"/annalist/sitedata",
            TestBaseDir)
        # testsite = Site(TestBaseUri, TestBaseDir)
        # testsite.generate_site_jsonld_context()
        # develsite = Site("http://localhost:8000/annalist/", TestBaseDir)
        return

# End.



"""
Annalist application
"""

__author__        = "Graham Klyne"
__author_email__  = "GK@ACM.ORG"
__copyright__     = "Copyright 2014, G. Klyne"
__license__       = "MIT (http://opensource.org/licenses/MIT)"

# https://docs.djangoproject.com/en/dev/ref/applications/
default_app_config = 'annalist.apps.AnnalistConfig'

"""
Software version number.

Even sub-releases are stable versions, except for critical bug-fixes
(designated by a patch letter at the end of the version string).

Odd sub-releases are work-in-progress, and may change from day-to-day.
"""
__version__        = "0.1.22"   # Software version number (odd = unstable)

"""
Data compatibility version: this is saved with collection data, and is used to
detect attempts to open newer data with older software.  It should be updated 
to the current version number whenever software updates would cause data to be
created that cannot be read by older versions of software.  (There is a general 
presumption that new software can read older data, and migrate it where necessary.)

Not all software updates create data that cannot be read by older software: for 
these updates the __version_data__ value can be left unchanged.
"""
__version_data__   = "0.1.22"   # Data compatibility version number

# End.
